@@ -5,6 +5,7 @@
 # backend_base.py: provides a baseclass for backend classes, and defines the API they implement.
 import os
 import inspect
+from time import time
 from typing import List
 from interface import implements
 
@@ -237,10 +238,12 @@ class BackendBase(implements(BackendInterface)):
             self.blobfuse_installed = True
 
     def create_blobfuse_commands(self, storage_name, storage_key, sudo_available, mount_requests, install_blobfuse, 
-            use_username=True, use_allow_other=True, nonempty=False, tmpbase = "$HOME"):
+            use_username=True, use_allow_other=True, nonempty=False, tmpbase="$HOME", args=None):
         username = "$USER"
         cmds = []
         sudo = "sudo " if sudo_available else ""
+        mount_retry_count = args["mount_retry_count"]
+        mount_retry_interval = args["mount_retry_interval"]
 
         # for each mount request
         for md in mount_requests:
@@ -279,8 +282,22 @@ class BackendBase(implements(BackendInterface)):
             self.append(cmds, "chmod 600 {}".format(fn_config))
             self.append(cmds, "blobfuse -v")
 
-            self.append(cmds, "{}blobfuse {} --tmp-path={}  --config-file={} {} -o attr_timeout=240 -o entry_timeout=240 -o negative_timeout=120 {} {}" \
-            .format(sudo, mnt_dir, tmp_dir, fn_config, readonly_opt, allow_other, nonempty_opt))
+            # need to wrap this command with retry logic (similiar to docker run)
+            #     pull_cmd = '''timeout {} bash -c "until {}; do '''.format(timeout, constants.FN_DOCKER_PULL_SH) + \
+            #         '''echo XT: docker pull failed.  retrying...; sleep 60; done" '''
+
+            blobfuse_cmd = "{}blobfuse {} --tmp-path={}  --config-file={} {} -o attr_timeout=240 -o entry_timeout=240 -o negative_timeout=120 {} {}" \
+                .format(sudo, mnt_dir, tmp_dir, fn_config, readonly_opt, allow_other, nonempty_opt)
+
+            if mount_retry_count:
+                # retry the blobfuse command as per count/interval
+                range = "{1.." + str(mount_retry_count) + "}"
+                retry_cmd = "for i in {}; do {} && break || echo error in blobfuse mount: sleeping... ; sleep {}; echo retrying blobfuse mount...; done".format(range, blobfuse_cmd, mount_retry_interval)
+                self.append(cmds, retry_cmd)
+
+            else:       
+                # normal blobfuse cmd (no retries)     
+                self.append(cmds, blobfuse_cmd)
 
             #self.append(cmds, "echo just ran blobfuse, here is ls -l on mnt_dir", echo=False)
             self.append_dir(cmds, mnt_dir)
@@ -350,7 +367,7 @@ class BackendBase(implements(BackendInterface)):
 
             requests = [ {"container": container, "mnt_dir": mnt_path, "readonly": not is_writable} ]
             sub_cmds = self.create_blobfuse_commands(storage_name, storage_key, sudo_available, requests, install_blobfuse=install_blobfuse,
-                use_username=use_username, use_allow_other=use_allow_other, nonempty=nonempty, tmpbase=tmpbase)
+                use_username=use_username, use_allow_other=use_allow_other, nonempty=nonempty, tmpbase=tmpbase, args=args)
             cmds += sub_cmds
 
     def process_action(self, cmds, action, mnt_path, container, store_data_dir, env_var_name, is_writable,
@@ -845,7 +862,7 @@ class BackendBase(implements(BackendInterface)):
 
         # run controller or user's cmd
         user_cmd = " ".join(user_parts)
-        fn_wrapped = self.create_wrapper_and_inner(cmds, user_cmd, args)
+        fn_wrapped = self.create_wrapper_and_inner(cmds, user_cmd, sudo_available, args)
 
         return fn_wrapped
 
@@ -1024,6 +1041,11 @@ class BackendBase(implements(BackendInterface)):
                 src_after = "$XT_NODE_DIR/after"
                 cmd = "xt upload {} {} --feedback=0".format(src_after, store_after)
                 self.append(cmds, cmd)
+
+        # if True:
+        #     # DEBUG: keep each node alive for 7 days for Azure Batch team to debug it
+        #     self.append(cmds, "sleep 7d")
+
         
     def export_now_to_var(self, cmds, var_name):
         '''
@@ -1046,19 +1068,58 @@ class BackendBase(implements(BackendInterface)):
         else:
             self.append_export(cmds, var_name, '$({})'.format(now_cmd), fix_value=False)
 
-    def append_pre_docker_cmds(self, cmds):
+    def append_pre_docker_cmds(self, cmds, args):
         if not self.gen_for_windows:
+            # show name of system
             self.append(cmds, "hostnamectl")
 
+            self.append_title(cmds, "CPU report:", True)
+            self.append(cmds, "lscpu")
+
+            self.append_title(cmds, "MEMORY report:", True)
+            self.append(cmds, "free -mh")
+
+            self.append_title(cmds, "GPU report:", True)
+            # show welcome msg (identifies DSVM)
+            self.append(cmds, "cat /etc/motd")
+
+            # show GPU installed
+            #self.append(cmds, "sudo lshw -C display | grep product")
+            self.append(cmds, "sudo lshw -C display")
+
+            # show GPU installed
+            self.append(cmds, "lspci -v")
+    
+            # several techinques to show version of NVIDIA DRIVERS
+            self.append(cmds, "whereis nvidia")
+            # self.append(cmds, 'grep "X Driver" /var/log/Xorg.0.log')
+            # self.append(cmds, 'modinfo /usr/lib/modules/$(uname -r)/kernel/drivers/video/nvidia.ko | grep ^version')
+            #self.append(cmds, "nvcc --version")
+
         # debug nvidia drivers/toolkit combinations
-        self.append(cmds, "nvcc --version")
         self.append(cmds, "nvidia-smi")
 
-        # TORCH version (is GPU available to torch?)
-        target_cmd = '''python -c "import torch; print('PyTorch:'.ljust(15) + ' ' + torch.__version__ + ', CUDA available: ' + str(torch.cuda.is_available()))"'''
-        self.append_package_test(cmds, "torch", target_cmd, "PyTorch")
+        if self.get_name() == "batch":
+            # run script to extract Azure Batch NODE and POOL ids
+            ws_id = args["workspace"]
+            job_id = args["job_id"]
 
-    def create_wrapper_and_inner(self, cmds, user_cmd, args):
+            # get BATCH credentials
+            service = self.compute_def["service"]
+            batch_creds = self.config.get_service(service)
+            batch_key = batch_creds["key"]
+            batch_url = batch_creds["url"]
+
+            batch_job_id = self.make_batch_job_id(ws_id, job_id)
+
+            self.append(cmds, "echo run script to extract Azure Batch NODE and POOL ids > /dev/null")
+            self.append(cmds, "python {} {} {} {} {} {}".format(constants.FN_BATCH_NODE_ID, batch_job_id, self.node_index, service, batch_url, batch_key))
+
+        # # TORCH version (is GPU available to torch?)
+        # target_cmd = '''python -c "import torch; print('PyTorch:'.ljust(15) + ' ' + torch.__version__ + ', CUDA available: ' + str(torch.cuda.is_available()))"'''
+        # self.append_package_test(cmds, "torch", target_cmd, "PyTorch")
+
+    def create_wrapper_and_inner(self, cmds, user_cmd, sudo_available, args):
 
         # debug
         self.append_dir(cmds)
@@ -1069,6 +1130,9 @@ class BackendBase(implements(BackendInterface)):
 
         self.export_now_to_var(cmds, "XT_POST_START_TIME")
 
+        docker_cmd = args["docker_cmd"]
+        timeout = args["docker_pull_timeout"]
+
         # append cmd(s) to download service and xt logs
         bootstrap_dir = args["bootstrap_dir"]
         self.add_log_upload_cmds(cmds, args)
@@ -1082,11 +1146,21 @@ class BackendBase(implements(BackendInterface)):
         # remove empty cmds
         cmds = [cmd for cmd in cmds if cmd]
 
+        # construct SLEEP CMD to randomly delay start of node execution
+        node_count = args["node_count"]
+        node_delay = args["node_delay"]
+
+        if node_delay and node_count > 1:
+            value = utils.shell_time_str_to_secs(node_delay)
+            sleep_cmd = "sleep $((RANDOM % {}))".format(value)
+        else:
+            sleep_cmd = None
+
         # for more reliable and easier-to-debug operation, write cmds to a .sh/.bat file (in bootstrap_dir)
-        docker_cmd = args["docker_cmd"]
         if docker_cmd:
             # write docker cmds to WRAPPED script (pool backend)
             docker_cmds = []
+
             # we need to generate for our native environment for the docker cmds
             self.gen_for_windows = self.is_windows
             index = 0
@@ -1096,31 +1170,102 @@ class BackendBase(implements(BackendInterface)):
                 index=1
 
             self.append_title(docker_cmds, 'START OF XT-LEVEL PROCESSING', double=True, index=index)
-            self.append_pre_docker_cmds(docker_cmds)
+
+            if sleep_cmd:
+                self.append(docker_cmds, sleep_cmd)
+
+            # # if service type is batch and using K80 GPU, we must install old drivers (on top of Ubuntu 18+)
+            # if self.get_name() == "batch" and self.compute_def["vm-size"] == "Standard_NC6":
+            #     # remove old NVIDIA drivers and CUDA
+            #     self.append(docker_cmds, "echo REMOVE old nvidia drivers and cuda")
+            #     self.append(docker_cmds, "sudo apt-get -y remove nvidia*")
+            #     self.append(docker_cmds, 'sudo apt-get -y --purge remove "*cublas*" "cuda*" "nsight*"')
+            #     self.append(docker_cmds, 'sudo apt-get -y --purge remove "*nvidia*"')
+
+            #     # add new NVIDIA drivers and CUDA
+            #     self.append(docker_cmds, "echo INSTALL new nvidia drivers and cuda")
+            #     self.append(docker_cmds, 'wget https://developer.download.nvidia.com/compute/cuda/repos/ubuntu1804/x86_64/cuda-ubuntu1804.pin')
+            #     self.append(docker_cmds, 'sudo mv cuda-ubuntu1804.pin /etc/apt/preferences.d/cuda-repository-pin-600')
+            #     self.append(docker_cmds, 'wget https://developer.download.nvidia.com/compute/cuda/11.4.2/local_installers/cuda-repo-ubuntu1804-11-4-local_11.4.2-470.57.02-1_amd64.deb')
+            #     self.append(docker_cmds, 'sudo dpkg -i cuda-repo-ubuntu1804-11-4-local_11.4.2-470.57.02-1_amd64.deb')
+            #     self.append(docker_cmds, 'sudo apt-key add /var/cuda-repo-ubuntu1804-11-4-local/7fa2af80.pub')
+            #     self.append(docker_cmds, 'sudo apt-get update')
+            #     self.append(docker_cmds, 'sudo apt-get -y install cuda')
+
+            self.append_pre_docker_cmds(docker_cmds, args)
 
             self.append_title(docker_cmds, "START DOCKER CONTAINER:")
             self.append_dir(docker_cmds)
 
-            # start docker service (in case it is not already running)
-            if self.gen_for_windows:
-                # don't do this for windows (screws up docker operation)
-                #self.append(docker_cmds, "net start com.docker.service 2> nul")
-                pass
-            else:                
-                self.append(docker_cmds, "sudo service docker start ")
-
             login_cmd = args["docker_login_cmd"]
             docker_name = args["docker"]
     
-            if login_cmd:
-                self.append(docker_cmds, login_cmd)
+            self.append(docker_cmds, 'test $(which nvidia-docker) && export DOCKER_RUN="nvidia-docker run"' + 
+                ' || export DOCKER_RUN="docker run --gpus all"')
 
-            if not self.is_windows:
-                # NOTE: we must set DOCKER_RUN before we can execute docker_cmd
-                # different versions of docker require different approaches
-                self.append(docker_cmds, 'test $(which nvidia-docker) && export DOCKER_RUN="nvidia-docker run" || export DOCKER_RUN="docker run --gpus all"')
+            self.append(docker_cmds, "echo $DOCKER_RUN")
 
-            self.append(docker_cmds, docker_cmd)
+            self.export_now_to_var(docker_cmds, "XT_PULL_START_TIME")
+
+            if timeout:
+                image_name = args["full_docker_image"]
+                #sudo = "sudo " # if sudo_available else ""
+
+                pull_cmds = []
+
+                self.append(pull_cmds, 'test $(which nvidia-docker) && export DOCKER_NAME="nvidia-docker"' + 
+                    ' || export DOCKER_NAME="docker"')
+
+                # restart docker service
+                self.append(pull_cmds, "echo restarting docker service...", echo=False)
+                self.append(pull_cmds, "sudo systemctl daemon-reload")
+                self.append(pull_cmds, "sudo systemctl restart docker")
+
+                if login_cmd:
+                    self.append(pull_cmds, login_cmd)
+
+                # pull docker image
+                self.append(pull_cmds, "echo pulling docker image...", echo=False)
+                self.append(pull_cmds, "sudo $DOCKER_NAME pull {}".format(image_name))
+
+                # test docker image by running XT with minimum command
+                self.append(pull_cmds, "echo testing docker image by running 'xt --version' within it...", echo=False)
+                self.append(pull_cmds, "sudo $DOCKER_NAME run --rm {} xt --version".format(image_name))
+
+                # write PULL CMDS to FN_DOCKER_PULL_SH
+                fn_wrapped = self.write_cmds_to_file(pull_cmds, bootstrap_dir, self.is_windows, constants.FN_DOCKER_PULL_BAT, constants.FN_DOCKER_PULL_SH)
+                utils.copy_to_submit_logs(args, fn_wrapped)
+
+                pull_cmd = '''timeout {} bash -c "until {}; do '''.format(timeout, constants.FN_DOCKER_PULL_SH) + \
+                    '''echo XT: docker pull+test failed.  retrying...; sleep 60; done" '''
+
+                self.append(docker_cmds, pull_cmd)
+                self.append(docker_cmds, "docker_pull_status=$?", echo=False)
+                self.append(docker_cmds, "if [ $docker_pull_status -ne 0 ]; then", echo=False)
+                self.append(docker_cmds, 'echo "XT: ERROR - docker pull TIMED OUT; job aborted"', echo=False)
+                self.append(docker_cmds, "else", echo=False)
+                self.append(docker_cmds, 'echo "XT: docker pull+test SUCCEEDED!"', echo=False)
+                self.append(docker_cmds, docker_cmd)
+                self.append(docker_cmds, "fi", echo=False)
+
+            else:
+                # not using TIMEOUT option
+
+                # start docker service (in case it is not already running)
+                if self.gen_for_windows:
+                    # don't do this for windows (screws up docker operation)
+                    #self.append(docker_cmds, "net start com.docker.service 2> nul")
+                    pass
+                else:                
+                    self.append(docker_cmds, "sudo service docker start ")
+
+                if login_cmd:
+                    self.append(docker_cmds, login_cmd)
+
+                if not self.is_windows:
+                    # NOTE: we must set DOCKER_RUN before we can execute docker_cmd
+                    # different versions of docker require different approaches
+                    self.append(docker_cmds, docker_cmd)
 
             fn_wrapped = self.write_cmds_to_file(docker_cmds, bootstrap_dir, self.is_windows, constants.FN_WRAPPED_BAT, constants.FN_WRAPPED_SH)
             utils.copy_to_submit_logs(args, fn_wrapped)
@@ -1140,7 +1285,9 @@ class BackendBase(implements(BackendInterface)):
             #insert_index = 1 if self.gen_for_windows else 0
             pre_cmds = []
             self.append_title(pre_cmds, 'START of XT-level processing', double=True)
-            self.append_pre_docker_cmds(pre_cmds)
+            if sleep_cmd:
+                self.append(pre_cmds, sleep_cmd)
+            self.append_pre_docker_cmds(pre_cmds, args)
             if self.gen_for_windows:
                 cmds = [ cmds[0] ] + pre_cmds + cmds[1:]
             else:
@@ -1163,15 +1310,15 @@ class BackendBase(implements(BackendInterface)):
 
         return fn_path
 
-    def get_controller_run_cmd(self, external_controller_port=constants.CONTROLLER_PORT, is_aml=False):
+    def get_controller_run_cmd(self, external_controller_port=constants.CONTROLLER_PORT, is_aml=False, multi_run_hold_open=False):
 
         if self.gen_for_windows:
-            text = '''python -u -c "from xtlib.controller import run; run(port={}, is_aml={})" '''
+            text = '''python -u -c "from xtlib.controller import run; run(port={}, is_aml={}, multi_run_hold_open={})" '''
         else:
             # linux needs these escapes around the double quotes
-            text = '''python -u -c 'from xtlib.controller import run; run(port={}, is_aml={})'  '''
+            text = '''python -u -c 'from xtlib.controller import run; run(port={}, is_aml={}, multi_run_hold_open={})'  '''
 
-        cmd = text.format(external_controller_port, is_aml)
+        cmd = text.format(external_controller_port, is_aml, multi_run_hold_open)
         return cmd
 
     def add_pip_packages(self, cmds, pip_packages):
