@@ -4,15 +4,17 @@
 #
 # backend_batch.py: support for running a XT job on a 1-N Azure Batch Compute boxes.
 
+from calendar import c
 import os
 import io
 import sys
-import math
+import shutil
 import time
 import datetime
 import numpy as np
 from interface import implements
 from threading import Lock
+from datetime import timedelta
 
 from xtlib import utils
 from xtlib import errors
@@ -92,6 +94,9 @@ class AzureBatch(BackendBase):
         self.batch_client = None
         #console.print("blob_client=", blob_client)
 
+        expire_days = self.config.get("general", "storage-cert-days")
+        self.cert_expiration = datetime.datetime.utcnow() + datetime.timedelta(hours=expire_days*24)
+
     def get_name(self):
         return "batch"
 
@@ -116,6 +121,7 @@ class AzureBatch(BackendBase):
         # local or POOL of vm's
         fn_wrapped = None     # we use same cmd on each box/job
         username = args["username"]
+        nowrap = args["nowrap"]
 
         for i, box_runs in enumerate(job_runs):
             node_id = "node" + str(i)
@@ -130,40 +136,54 @@ class AzureBatch(BackendBase):
             run_specs = br["run_specs"]
             cmd_parts = run_specs["cmd_parts"]
 
-            # just create the fn_wrapped for the first node (but use for all nodes)
-            if i == 0:
-                node_cmd = " ".join(cmd_parts)
+            if nowrap:
+                # run user's script directly
+                fn_direct = os.path.basename(args["script"])
+                #run_specs["cmd_parts"] = ["echo i was here; ls -lt; echo bye now"]   # [fn_direct]
+                cmd = "/bin/bash --login -c 'echo starting script: {}; {}; echo script ended'".format(fn_direct, fn_direct)
+                run_specs["cmd_parts"] = [cmd]
+            
+            else:
+                # just create the fn_wrapped for the first node (but use for all nodes)
+                if i == 0:
+                    node_cmd = " ".join(cmd_parts)
 
-                fn_wrapped = self.wrap_cmd_for_batch(node_cmd, node_id, store_data_dir, data_action, data_writable, 
-                    store_model_dir, model_action, model_writable, storage_name, storage_key, snapshot_dir, 
-                    box_secret, args)
+                    fn_wrapped = self.wrap_cmd_for_batch(node_cmd, node_id, store_data_dir, data_action, data_writable, 
+                        store_model_dir, model_action, model_writable, storage_name, storage_key, snapshot_dir, 
+                        box_secret, args)
 
-            # NOTE: we pass the NODE_INDEX and RUN_NAME as parameters to the fn_wrapped script
-            # so that we can share a single script among all nodes
-            wrapped_parts = ['/bin/bash', '--login', '{} {} {}'.format(os.path.basename(fn_wrapped), i, run_name)]
-            run_specs["cmd_parts"] = wrapped_parts
+                # NOTE: we pass the NODE_INDEX and RUN_NAME as parameters to the fn_wrapped script
+                # so that we can share a single script among all nodes
+                wrapped_parts = ['/bin/bash', '--login', '{} {} {}'.format(os.path.basename(fn_wrapped), i, run_name)]
+                run_specs["cmd_parts"] = wrapped_parts
 
-            merge_batch_logs = args["merge_batch_logs"]
-            if merge_batch_logs:
-                # recompose this command to pass along all script arguments
-                wrapped_parts = ['/bin/bash', '--login', '{} $*'.format(os.path.basename(fn_wrapped))]
-    
-                # create a STDBOTH.tmp that merged stdout and stderr
-                node_cmd = "{} > ../stdboth.tmp 2>&1 && cp ../stdboth.tmp ../stdboth.txt".format(" ".join(wrapped_parts))
+                merge_batch_logs = args["merge_batch_logs"]
+                if merge_batch_logs:
+                    # recompose this command to pass along all script arguments
+                    wrapped_parts = ['/bin/bash', '--login', '{} $*'.format(os.path.basename(fn_wrapped))]
+        
+                    # create a STDBOTH.txt that merged stdout and stderr
+                    #node_cmd = "{} > ../stdboth_tmp.txt 2>&1 && cp ../stdboth_tmp.txt ../stdboth.txt".format(" ".join(wrapped_parts))
+                    node_cmd = "{} > ../stdboth.txt 2>&1".format(" ".join(wrapped_parts))
 
-                # create a helper file to call our node cmd
-                fn_helper = snapshot_dir + "/" + constants.FN_BATCH_HELPER
-                file_utils.write_text_file(fn_helper, node_cmd)
+                    # create a helper file to call our node cmd
+                    fn_helper = snapshot_dir + "/" + constants.FN_BATCH_HELPER
+                    file_utils.write_text_file(fn_helper, node_cmd)
 
-                run_specs["cmd_parts"] = ["bash", constants.FN_BATCH_HELPER, str(i), run_name]
+                    run_specs["cmd_parts"] = ["bash", constants.FN_BATCH_HELPER, str(i), run_name]
 
-                console.diag(run_specs)
+                    console.diag(run_specs)
+
+        # copy node_id to snapshot dir
+        fn_from = file_utils.get_xtlib_dir() + "/backends/batch_node_id.py" 
+        fn_to = snapshot_dir + "/" + constants.FN_BATCH_NODE_ID
+        shutil.copyfile(fn_from, fn_to)
 
     def wrap_cmd_for_batch(self, user_cmd, node_id, store_data_dir, data_action, data_writable, 
         store_model_dir, model_action, model_writable, storage_name, storage_key, snapshot_dir, box_secret, args):
 
         cmds = []
-
+        
         gen_for_windows = False
         self.capture_setup_cmds = args["capture_setup_cmds"]
 
@@ -227,7 +247,7 @@ class AzureBatch(BackendBase):
             actions, data_action, model_action, store_data_dir, store_model_dir, data_writable, model_writable,
             use_username=False, use_allow_other=False, install_blobfuse=install_blobfuse, args=args)
 
-        fn_wrapped = self.create_wrapper_and_inner(cmds, user_cmd, args)
+        fn_wrapped = self.create_wrapper_and_inner(cmds, user_cmd, sudo_available, args)
         return fn_wrapped
 
     def print_things(self, things, columns):
@@ -329,7 +349,7 @@ class AzureBatch(BackendBase):
         self.prep_all_nodes(job_id, job_runs, num_nodes, use_low_pri, using_hp, experiment, 
             static_run_cmds, node_records, workspace, service_info_by_node, merge_batch_logs, args)
 
-        # "hold" is now used to hold open the xt controller task AND the created pool
+        # "hold" is used to hold open the created pool
         auto_pool = not args["hold"]
 
         fb.feedback("submitting job to batch")
@@ -462,6 +482,7 @@ class AzureBatch(BackendBase):
 
         self.batch_job_id = batch_job_id
         self.pool_id = "_pool_{}_{}_".format(workspace, job_id)
+        self.xt_job_id = job_id
 
         self.vm_size = vm_size
         self.azure_image = vm_image 
@@ -632,14 +653,22 @@ class AzureBatch(BackendBase):
         if not props:
             errors.config_error("No config file entry found in [azure-batch-images] section for azure-image=" + self.azure_image)
 
-        publisher = props["publisher"]
-        offer = props["offer"]
-        sku = props["sku"]
-        version = props["version"]
-        node_agent_sku_id = props["node-agent-sku-id"]
+        publisher =  utils.safe_value(props, "publisher")
+        offer =  utils.safe_value(props, "offer")
+        sku =  utils.safe_value(props, "sku")
+        version =  utils.safe_value(props, "version")
+        node_agent_sku_id =  utils.safe_value(props, "node-agent-sku-id")
+        custom_image_id = utils.safe_value(props, "custom-image-id")
 
-        img_ref = batchmodels.ImageReference(publisher=publisher, offer=offer, sku=sku, version=version)
-        vmc = batchmodels.VirtualMachineConfiguration(image_reference=img_ref, node_agent_sku_id=node_agent_sku_id)
+        if custom_image_id:
+            # supply a custom OS image for each node in the pool
+            img_ref = batchmodels.ImageReference(virtual_machine_image_id=custom_image_id)
+            vmc = batchmodels.VirtualMachineConfiguration(image_reference=img_ref, node_agent_sku_id=node_agent_sku_id)
+
+        else:
+            # use a Microsoft published image
+            img_ref = batchmodels.ImageReference(publisher=publisher, offer=offer, sku=sku, version=version)
+            vmc = batchmodels.VirtualMachineConfiguration(image_reference=img_ref, node_agent_sku_id=node_agent_sku_id)
 
         # support for running a user-specified custom container
         target = self.compute
@@ -680,8 +709,7 @@ class AzureBatch(BackendBase):
 
         if self.auto_pool:
 
-            # this was working, but looks like it stopped (as of 11/22/2019 - rfernand2)
-            # create a dynamically allocated pool
+            # create a dynamically allocated pool  (name will be GUID assigned by Azure Batch)
             #dynamic_resize = True
             dynamic_resize = False
 
@@ -717,10 +745,13 @@ class AzureBatch(BackendBase):
                     #max_tasks_per_node = max_tasks_per_node,
                     enable_inter_node_communication=enable_inter_node_communication)
 
-            auto_pool = batch.models.AutoPoolSpecification(pool_lifetime_option="job", keep_alive=False, pool=pool_spec)
+            auto_pool = batch.models.AutoPoolSpecification(pool_lifetime_option="job", keep_alive=False, pool=pool_spec, 
+                auto_pool_id_prefix=self.xt_job_id)
+
             pool_info = batch.models.PoolInformation(auto_pool_specification= auto_pool)
         else:
-            # create a statically allocated pool
+            # create a statically allocated pool that must be released by the user
+            # for this pool, we can provide a name to easily associate it with the job
             new_pool = batch.models.PoolAddParameter(id=self.pool_id,
                 virtual_machine_configuration=vmc,
                 vm_size=self.vm_size,
@@ -729,6 +760,7 @@ class AzureBatch(BackendBase):
                 target_low_priority_nodes=self.num_nodes if self.use_low_pri else 0,
                 #max_tasks_per_node = max_tasks_per_node,
                 enable_auto_scale=False,                # don't resize to 0 or we won't be able to connect to node
+                resize_timeout=timedelta(minutes=30),                      # allow 30 mins to allocate all needed VMs
                 enable_inter_node_communication=enable_inter_node_communication)
 
             self.batch_client.pool.add(new_pool)
@@ -757,12 +789,10 @@ class AzureBatch(BackendBase):
         
         self.blob_client.create_container(dest_container_name, fail_on_exist=False)
 
-        expire_days = self.config.get("general", "storage-cert-days")
-
         # create an SAS for writing to the container
         sas_token = self.blob_client.generate_container_shared_access_signature(dest_container_name, 
             permission=azureblob.ContainerPermissions.WRITE,
-            expiry=datetime.datetime.utcnow() + datetime.timedelta(hours=expire_days*24))
+            expiry=self.cert_expiration)
 
         # secret trick: construct SAS URL for the container
         out_container_url = "https://{}.blob.core.windows.net/{}?{}".format(self.store_name, dest_container_name, sas_token)
@@ -1164,7 +1194,8 @@ class AzureBatch(BackendBase):
 
         raise RuntimeError('could not read task data from stream')
 
-    def convert_blobs_to_resource_files(self, container_name, blob_names, file_names, writable=False, expire_hours=24):
+    def convert_blobs_to_resource_files(self, container_name, blob_names, file_names, writable=False):
+
         if writable:
             permission = azureblob.BlobPermissions.WRITE
         else:
@@ -1175,10 +1206,8 @@ class AzureBatch(BackendBase):
         #for blob_name, file_name in zip(blob_names, file_names):
         for i, blob_name in enumerate(blob_names):
             # create a security token to allow anonymous access to blob
-            expire_date = expiry=datetime.datetime.utcnow() + datetime.timedelta(hours=expire_hours)
-
             sas_token = self.blob_client.generate_blob_shared_access_signature(container_name,
-                blob_name, permission=permission, expiry=expire_date)
+                blob_name, permission=permission, expiry=self.cert_expiration)
 
             # convert SAS to URL
             sas_url = self.blob_client.make_blob_url(container_name, blob_name, sas_token=sas_token)
@@ -1460,7 +1489,8 @@ class AzureBatch(BackendBase):
 
             try:
                 # stdboth is a .TMP file when it is read live
-                live_log_name = "stdboth.tmp" if log_name == "stdboth.txt" else log_name
+                #live_log_name = "stdboth_tmp.txt" if log_name == "stdboth.txt" else log_name
+                live_log_name = "stdboth.txt" 
 
                 stream = self.batch_client.file.get_from_task(batch_job_id, task.id, file_path=live_log_name, 
                     file_get_from_task_options=gft_opts)
