@@ -395,7 +395,7 @@ class PoolBackend(BackendBase):
 
     # BACKEND API
     def read_log_file(self, service_node_info, log_name, start_offset=0, end_offset=None, 
-        encoding='utf-8', use_best_log=True):
+        encoding='utf-8', use_best_log=True, log_source=None):
         '''
         returns subset of specified log file as text.
         '''
