@@ -236,7 +236,7 @@ def get_box_list(core, job_id=None, explicit_boxes_only=False, box=None, pool=No
 def get_pool_info(core, pool_name, args):
 
     # get a COPY of the target dict
-    pool_info = dict(core.config.get_compute_def(pool_name))
+    pool_info = dict(core.config.get_target_def(pool_name))
     config = core.config
 
     if isinstance(pool_info, list):
