@@ -28,11 +28,12 @@ def calc_size_of_files(filenames):
 
 def capture_before_files_zip(store, source_dir, ws_name=None, run_name=None, extra_files = [], rerun_name=None, 
         job_id=None,  omit_files=None, zip_before="none", store_dest="before", remove_prefix_len=None, 
-        upload_type=None, service_type=None):
+        upload_type=None, service_type=None, args=None):
     
     copied_files = []
     started = time.time()
     feedback = True   # (upload_type != "bootstrap")
+    nowrap = args["nowrap"]
 
     if rerun_name:
         if "/" in rerun_name:
@@ -76,11 +77,15 @@ def capture_before_files_zip(store, source_dir, ws_name=None, run_name=None, ext
 
             if service_type in ["batch", "itp", "aml"]:
                 # add a copy of bootstrap files in parallel to the .zip file so they are available when we start to run
-                # bootstrap_files = [constants.FN_WRAPPED_SH, constants.FN_WRAPPED_BAT, constants.FN_INNER_SH, constants.FN_INNER_BAT, 
-                #     constants.FN_EV, constants.PY_RUN_CONTROLLER, constants.FN_MULTI_RUN_CONTEXT]
                 if service_type == "batch":
                     bootstrap_files = [constants.FN_WRAPPED_SH, constants.FN_WRAPPED_BAT, constants.FN_INNER_SH, \
+                        constants.FN_DOCKER_PULL_SH, constants.FN_DOCKER_PULL_BAT, constants.FN_BATCH_NODE_ID,  \
                         constants.FN_INNER_BAT, constants.FN_EV, constants.FN_BATCH_HELPER]      
+
+                    if nowrap:
+                        fn_script = args["script"]
+                        bootstrap_files.append(fn_script)   
+                        bootstrap_files.append("code/mnist.py")
                 else:
                     bootstrap_files = [constants.FN_WRAPPED_SH, constants.FN_WRAPPED_BAT, constants.FN_EV, 
                         constants.PY_RUN_CONTROLLER, constants.FN_MULTI_RUN_CONTEXT, constants.FN_AML_SHIM]      
@@ -166,7 +171,7 @@ def unzip_before_if_found(dest_dir, remove_after=True, silent=False):
 
     return names
 
-def download_run(store, ws_name, run_name, dest_dir):
+def download_run(store, ws_name, run_name, dest_dir, others_dict):
     '''
        - first, download entire RUN store
        - then, if BEFORE files are in job, download them
@@ -176,16 +181,32 @@ def download_run(store, ws_name, run_name, dest_dir):
     files += store.download_files_from_run(ws_name, run_name, "**", dest_dir)
     
     job_id = store.get_job_id_of_run(ws_name, run_name)
+    node_id = store.get_node_id_of_run(ws_name, run_name)
 
-    # this will issue no errors if no files are found
-    before_dir = dest_dir + "/before"
-    before_files = store.download_files_from_job(ws_name, job_id, "before/**", before_dir)
+    # download BEFORE files of job, if needed
+    before_key = job_id + "/before"
 
-    zip_files = unzip_before_if_found(before_dir + "/code", silent=True)
-    if zip_files:
-        files += zip_files
-    else:
-        files += before_files
+    if not before_key in others_dict:
+        before_dir = dest_dir + "/before"
+        before_files = store.download_files_from_job(ws_name, job_id, "before/**", before_dir)
+
+        zip_files = unzip_before_if_found(before_dir + "/code", silent=True)
+        if zip_files:
+            files += zip_files
+        else:
+            files += before_files
+
+        others_dict[before_key] = 1
+
+    # download NODE files of job, if needed
+    node_key = job_id + "/nodes/{}".format(node_id)
+    if not node_key in others_dict:
+        from_node = "nodes/{}/**".format(node_id)
+        to_node = dest_dir + "/nodes/{}".format(node_id)
+        node_files = store.download_files_from_job(ws_name, job_id, from_node, to_node)
+
+        files += node_files
+        others_dict[node_key] = 1    
 
     return files
 
