@@ -52,11 +52,28 @@ class Console():
         text = self.print_to_string(*objects, sep=sep, end=end)
         self.early_output.append( (target_level, text) )
 
-    def print(self, *objects, sep=' ', end='\n', file=sys.stdout, flush=False):
+    def print(self, *objects, sep=' ', end='\n', file=sys.stdout, flush=False, exit_time=0):
         if self.level == None:
             self.early_print("normal", *objects, sep=sep, end=end, file=file, flush=flush)
         elif self.level != "none":
-            print(*objects, sep=sep, end=end, file=file, flush=flush)
+            #print(*objects, sep=sep, end=end, file=file, flush=flush)
+            
+            text = self.print_to_string(*objects, sep=sep, end="")
+
+            if self.level == "timing":
+                # JIT import to prevent import cycles here
+                from .helpers.feedbackParts import feedback
+                if self.level and feedback.in_feedback:
+                    feedback.stop_feedback()
+
+                elapsed = time.time() - self.xt_started + exit_time
+                delta = time.time() - self.xt_last_time + exit_time
+                self.xt_last_time = time.time()
+
+                # add timing info at start of output
+                text = "[{:.2f}, +{:.2f}]: {}".format(elapsed, delta, text)
+
+            print(text, end=end, file=file, flush=flush)
 
         if self.capturing_output:
             text = sep.join([str(obj) for obj in objects]) + end
@@ -93,7 +110,7 @@ class Console():
     def set_level(self, level):
         '''
         args:
-            level: one of these strings: none, normal, diagnostics, and detail 
+            level: one of these strings: none, normal, timing, diagnostics, and detail 
         '''
         self.level = level
 
@@ -106,7 +123,7 @@ class Console():
         if level and self.early_output:
             # process early output now
             for target, text in self.early_output:
-                if level == "normal" and target == "normal":
+                if level in ["normal", "timing"] and target == "normal":
                     all_text += text
                 elif level == "diagnostics" and target in ["normal", "diagnostics"]:
                     all_text += text
