@@ -6,6 +6,7 @@
 import sys
 import os
 import json
+import xxlimited
 import rpyc
 import copy
 import time
@@ -24,6 +25,7 @@ from rpyc.utils.authenticators import SSLAuthenticator
 from xtlib.utils import log_info, log_title, log_info_to_text
 
 # xtlib 
+from xtlib import store_utils
 from xtlib.helpers.bag import Bag
 from xtlib.console import console
 from xtlib.run_info import RunInfo
@@ -178,6 +180,7 @@ class XTController(rpyc.Service):
         self.mirror_workers = []
         self.running_jobs = {}
         self.next_run_list_index = 0
+        self.stop_processing_runs = False
 
         self.box_secret = os.getenv("XT_BOX_SECRET")
         self.node_id = os.getenv("XT_NODE_ID")
@@ -282,6 +285,7 @@ class XTController(rpyc.Service):
         context_dict = dd["runs"][0]
         context = utils.dict_to_object(context_dict)
         parent_run_name = context.run_name
+        self.sleep_on_exit = context.sleep_on_exit
 
         self.mrc_context = context
         self.ws_name = context.ws
@@ -298,16 +302,38 @@ class XTController(rpyc.Service):
         # is this node restarting after a low-priority preemption (or a simulated one)?
         # check the existance of the node.log file for this node 
         node_path = "nodes/{}/node.log".format(self.node_id)
+        log_info("self.ws_name", self.ws_name)
+        log_info("self.job_id", self.job_id)
+        log_info("node_path", node_path)
 
-        if store.does_job_file_exist(self.ws_name, self.job_id, node_path):
+        node_log_exists = store.does_job_file_exist(self.ws_name, self.job_id, node_path)
+        log_info("node_log_exists", node_log_exists)
+
+        '''
+        NOTE: For some reason, we cannot rely on the presence of the node log file to detect
+        if this node has restarted (node_log_exists=False when we know a node has been restarted), so we will reply on the node_stats (database) info.
+        '''
+        pull_start_time_str = os.getenv("XT_PULL_START_TIME", None)
+        prep_start_time_str = os.getenv("XT_PREP_START_TIME")
+
+        # TODO: add pull_start_time_str to API to compute PULL_ELAPSED time
+        node_restarted = db.node_start(self.ws_name, self.job_id, node_index=self.node_index, node_restart=node_log_exists, prep_start_str=prep_start_time_str)
+
+        if node_restarted:
+            log_info("node RESTART detected", True)
             self.node_restart = True
             text = self.make_event_text("restarted")
             store.append_job_file(self.ws_name, self.job_id, node_path, text)
-            print("=========> NODE RESTART DETECTED <==============")
         else:
+            log_info("node RESTART detected", False)
+            self.node_restart = False
             text = self.make_event_text("started")
             store.append_job_file(self.ws_name, self.job_id, node_path, text)
 
+        # debug
+        after_node_log_exists = store.does_job_file_exist(self.ws_name, self.job_id, node_path)
+        log_info("after_node_log_exists", after_node_log_exists)
+
         # log NODE_START / NODE_RESTART to job
         start_name = "node_restart" if self.node_restart else "node_start"
 
@@ -316,11 +342,7 @@ class XTController(rpyc.Service):
 
         # tell db the first time this job node starts running
         log_info("calling job_node_start", self.job_id)
-        db.job_node_start(self.ws_name, self.job_id, self.node_index, self.node_restart)
-
-        prep_start_time_str = os.getenv("XT_PREP_START_TIME")
-
-        db.node_start(self.ws_name, self.job_id, self.node_index, self.node_restart, prep_start_time_str)
+        db.job_node_start(self.ws_name, self.job_id, node_index=self.node_index, is_restart=self.node_restart)
 
         was_queued = []    # list of runs that were queued before we were restarted
 
@@ -339,10 +361,11 @@ class XTController(rpyc.Service):
         
         # simple scheduling
         self.run_indexes = list(range(context.first_run_index, 1 + context.last_run_index))
+        log_info("full run_indexes", self.run_indexes)
 
         if self.node_restart:
             # remove any runs that have completed (don't rely on v1/v2 child run name differences)
-            filter_dict = {"ws_name": self.ws_name, "job_id": self.job_id, "node_index": self.node_index, \
+            filter_dict = {"ws_name": self.ws_name, "job_id": self.job_id, "node_index": self.node_index, 
                 "end_id": {"$exists": True}}
 
             completed_run_docs = self.store.database.get_info_for_runs(self.ws_name, filter_dict, {"run_index": 1})
@@ -443,7 +466,14 @@ class XTController(rpyc.Service):
 
         # give other threads time to wrapup the processing of their runs before
         # we exit
-        time.sleep(2)    # wait for 2 secs for any bg thread cleanup
+        if self.sleep_on_exit:
+            log_info("controller sleeping", "{} ...".format(self.sleep_on_exit))
+            # convert from shell time to number of seconds
+            sleep_in_secs = utils.shell_time_str_to_secs(self.sleep_on_exit)
+            time.sleep(sleep_in_secs)
+        else:
+            time.sleep(2)    # wait for 2 secs for any bg thread cleanup
+
         console.print("controller calling os._exit(0)...")
 
         # os._exit will exit all threads without running 'finally' blocks 
@@ -610,14 +640,32 @@ class XTController(rpyc.Service):
             # get index of next run for our node storage
             #console.print("self.next_run_list_index=", self.next_run_list_index)
 
-            if self.next_run_list_index < len(self.run_indexes):
+            if self.stop_processing_runs:
+                entry = None
+
+            elif context.search_type == "ccd":
+                # CCD runs are dynamically created on each node (not a fixed count)
+                if store_utils.STORAGE_FORMAT == "1":
+                    # each node allocates in own 0-relative run_index
+                    self.first_run_index = 1
+                    run_index = self.next_run_list_index
+                    self.next_run_list_index += 1
+
+                else:
+                    # let database allocate next job-level run_index
+                    run_index =  self.store.database.get_next_run_index(context.ws, context.job_id)
+
+                entry = {"status": "unstarted", "run_index": run_index}
+
+            elif self.next_run_list_index < len(self.run_indexes):
                 run_index = self.run_indexes[self.next_run_list_index]
                 self.next_run_list_index += 1
                 entry = {"status": "unstarted", "run_index": run_index}
+
             else:
                 entry = None
 
-            log_info("get_next_run_index",  entry)
+            log_info("entry",  entry)
 
             # log run index to JOB
             store.log_node_event(self.ws_name, self.job_id, self.node_id, 
@@ -625,7 +673,16 @@ class XTController(rpyc.Service):
 
             if entry:
                 run_index = entry["run_index"]
-                log_info("==> running INDEX", "{}/{}".format(run_index, context.total_run_count-1))
+
+                if context.search_type == "ccd":
+                    if context.total_run_count is None:
+                        limit = "unlimited"
+                    else:
+                        limit = context.total_run_count-1
+                else:
+                    limit = context.total_run_count-1
+
+                log_info("==> running INDEX", "{}/{}".format(run_index, limit))
 
                 # yes: CREATE CHILD
                 self.run_template(run_info, run_index, entry)
@@ -675,18 +732,20 @@ class XTController(rpyc.Service):
 
         # spawn child run from template
         child_info = self.spawn_child(run_info, run_index, entry)
-        child_name = child_info.run_name
+        if child_info:
 
-        # add to runs
-        self.add_to_runs(child_info)
+            # add to runs
+            self.add_to_runs(child_info)
 
-         # start normal run of CHILD
-        self.start_local_run(child_info, cmd_parts=child_info.cmd_parts)
-
-        if run_info.status == "queued":
-            # create a parent log event for "spawing"
-            store.log_run_event(context.ws, run_name, "status-change", {"status": "queued"})  
+            # start normal run of CHILD
+            self.start_local_run(child_info, cmd_parts=child_info.cmd_parts)
 
+            if run_info.status == "queued":
+                # create a parent log event for "spawing"
+                store.log_run_event(context.ws, run_name, "status-change", {"status": "queued"})  
+        else:
+            self.stop_processing_runs = True
+            
     def requeue_run(self, run_info):
         with queue_lock:
             self.queue.append(run_info)
@@ -724,67 +783,87 @@ class XTController(rpyc.Service):
         store = self.store
 
         log_info("spawn_child, parent", parent.run_name)
-
-        child_aml_run = None
-        child_name = self.store.database.get_child_name(entry, self.parent_run_name, self.first_run_index)
-
         log_info("child run_index", run_index)
-        log_info("created child", child_name)
-
-        # the logged value of search_type reflects if it was really used
-        if context.search_style in ["dynamic", "static"]:
-           search_type = context.search_type
-        else:
-            search_type = None
-
-        store.start_child_run(context.ws, parent.run_name, context.exper_name,
-            child_name=child_name, 
-            box_name=context.box_name, app_name=context.app_name, path=context.target_file,
-            from_ip=context.from_ip, from_host=context.from_host, sku=context.sku,
-            job_id=context.job_id, pool=context.pool, node_index=self.node_index, 
-            aggregate_dest=context.aggregate_dest,  
-            compute=context.compute, service_type=context.service_type, username=context.username, 
-            search_type=search_type, run_index=run_index)
 
         if context.search_style == "dynamic":
             # perform dynamic HPARAM search
-            log_info("doing dynamic HPARAM search for child", child_name)
-            cmd_parts = self.hparam_search.process_child_hparams(child_name, store, context, parent)
-        elif self.runsets:
-            # select a runset and apply to run
-            actual_ri = run_index % len(self.runsets)
-            log_info("getting runset for child=" + child_name, "index=" + str(actual_ri))
+            run_index_name = "run_index_" + str(run_index)
+            log_info("doing dynamic HPARAM search for child", run_index_name)
+    
+            arg_dict = self.hparam_search.process_child_hparams(run_index_name, store, context, parent)
+            if arg_dict:
+                # now its safe to create the child name
+                child_name = self.store.database.get_child_name(entry, self.parent_run_name, self.first_run_index)
+                log_info("created child", child_name)
 
-            runset = self.runsets[actual_ri]
-            log_info("runset for child", runset)
+                cmd_parts = self.hparam_search.apply_runset(arg_dict, context.cmd_parts, context, store, child_name)
+            else:
+                # we should stop processing runs now
+                cmd_parts = None
 
-            cmd_parts = list(context.cmd_parts)
-            cmd_parts = self.hparam_search.apply_runset(runset, cmd_parts, context, store, child_name)
-            log_info("runset applied to cmd_parts", cmd_parts)
+            hp_set = str(arg_dict)
         else:
-            # select the normal command for the run
-            log_info("normal child without HPARAM search", child_name)
-            cmd_parts = context.cmd_parts
+            child_name = self.store.database.get_child_name(entry, self.parent_run_name, self.first_run_index)
+            log_info("created child", child_name)
 
-        # must update context info
-        context.run_name = child_name
+            if self.runsets:
+                # select a runset and apply to run
+                actual_ri = run_index % len(self.runsets)
+                log_info("getting runset for child=" + child_name, "index=" + str(actual_ri))
 
-        # log run CMD
-        store.log_run_event(context.ws, child_name, "cmd", {"cmd": cmd_parts, "xt_cmd": context.xt_cmd}, job_id=context.job_id)  
+                runset = self.runsets[actual_ri]
+                log_info("runset for child", runset)
 
-        # for now, don't log context (contain private credentials and not clear if we really need it)
-        # for CHILD runs, record all "context" (from cmd line, user config, default config) in log (for audit/re-run purposes)
-        #store.log_run_event(context.ws, child_name, "context", context.__dict__)
+                cmd_parts = list(context.cmd_parts)
+                cmd_parts = self.hparam_search.apply_runset(runset, cmd_parts, context, store, child_name)
+                log_info("runset applied to cmd_parts", cmd_parts)
 
-        # get_client_context() should have set this correct for this parent/child run
-        #prep_script = context.child_prep_script     
-        #prep_script = None
+                hp_set = str(runset)
+            else:
+                # select the normal command for the run
+                log_info("normal child without HPARAM search", child_name)
+                cmd_parts = context.cmd_parts
+                hp_set = None
 
-        child_info = RunInfo(child_name, context.ws, cmd_parts, context.run_script, 
-            None, context, "running", True, parent_name=parent.run_name, mirror_close_func = self.stop_mirror_worker, 
-            node_id=self.node_id, run_index=run_index, store=self.store)
+        if cmd_parts:
+            # HP search succeeded
+            # the logged value of search_type reflects if it was really used
+            if context.search_style in ["dynamic", "static"]:
+                search_type = context.search_type
+            else:
+                search_type = None
 
-        parent.process_run_output("spawned: {}\n".format(child_name))
+            store.start_child_run(context.ws, parent.run_name, context.exper_name,
+                child_name=child_name, 
+                box_name=context.box_name, app_name=context.app_name, path=context.target_file,
+                from_ip=context.from_ip, from_host=context.from_host, sku=context.sku,
+                job_id=context.job_id, pool=context.pool, node_index=self.node_index, 
+                aggregate_dest=context.aggregate_dest,  
+                compute=context.compute, service_type=context.service_type, username=context.username, 
+                search_type=search_type, run_index=run_index, hp_set=hp_set)
+
+            # must update context info
+            context.run_name = child_name
+            context.node_index = self.node_index
+
+            # log run CMD
+            store.log_run_event(context.ws, child_name, "cmd", {"cmd": cmd_parts, "xt_cmd": context.xt_cmd}, job_id=context.job_id)  
+
+            # for now, don't log context (contain private credentials and not clear if we really need it)
+            # for CHILD runs, record all "context" (from cmd line, user config, default config) in log (for audit/re-run purposes)
+            #store.log_run_event(context.ws, child_name, "context", context.__dict__)
+
+            # get_client_context() should have set this correct for this parent/child run
+            #prep_script = context.child_prep_script     
+            #prep_script = None
+
+            child_info = RunInfo(child_name, context.ws, cmd_parts, context.run_script, 
+                None, context, "running", True, parent_name=parent.run_name, mirror_close_func = self.stop_mirror_worker, 
+                node_id=self.node_id, run_index=run_index, store=self.store)
+
+            parent.process_run_output("spawned: {}\n".format(child_name))
+        else:
+            child_info = None
 
         return child_info
  
@@ -849,6 +928,14 @@ class XTController(rpyc.Service):
         else:
             run_info.is_wrapped_up = True
 
+        context = run_info.context
+
+        if run_info.parent_name and context.search_style == "dynamic":
+            stop_controller = self.hparam_search.process_end_of_run(self.store, context)
+
+            if stop_controller:
+                self.stop_processing_runs = True
+
         log_info("end of", "exit handler")
 
     def on_connect(self, conn):
@@ -1101,15 +1188,6 @@ class XTController(rpyc.Service):
 
         self.start_local_run_core(run_info, cmd_parts, rundir, run_dir_index)
 
-        # try:
-        #     self.start_local_run_core(run_info, cmd_parts, rundir, run_index)
-        # except BaseException as ex:
-        #     logger.exception("Error in controller.start_local_run, ex={}".format(ex))
-        #     console.print("** Exception during start_local_run(): ex={}".format(ex))
-        #     self.exit_handler(run_info, False, called_from_thread_watcher=False)
-        #     # in controller code, always give up machine if we fail 
-        #     raise ex   
-
     def start_local_run_core(self, run_info, cmd_parts, rundir, rundir_index):
         '''
         Note: 
@@ -1250,7 +1328,7 @@ class XTController(rpyc.Service):
             log_info("WINDOWS script", fn_script)
         else:
             if not fn_script:
-                fn_script = self.fix_path("{}/run_app{}.sh".format(self.cwd,    rundir_index))
+                fn_script = self.fix_path("{}/run_app{}.sh".format(self.cwd, rundir_index))
             #utils.send_cmd_as_script_to_box(self, "localhost", flat_cmd, fn_script, prep_script, True)
             scriptor.write_script_file(run_script, fn_script, for_windows=False)
             log_info("LINUX script", fn_script)
@@ -1474,41 +1552,28 @@ class XTController(rpyc.Service):
                     result += run_info.get_summary_stats() + "\n"
         return result
 
-    def cancel_core(self, run_info, for_restart=False):
-        result = None
-        status = None
+    # def cancel_core(self, run_info, for_restart=False):
+    #     result = None
+    #     status = None
         
-        log_info("----> killing", "{}/{}".format(run_info.workspace, run_info.run_name))
+    #     log_info("----> killing", "{}/{}".format(run_info.workspace, run_info.run_name))
 
-        with queue_lock:
-            if run_info in self.queue:
-                self.queue.remove(run_info)
+    #     with queue_lock:
+    #         if run_info in self.queue:
+    #             self.queue.remove(run_info)
 
-        # log run KILLED event 
-        context = run_info.context
-        store = self.store
+    #     # log run KILLED event 
+    #     context = run_info.context
+    #     store = self.store
 
-        if not for_restart:
-            store.log_run_event(context.ws, run_info.run_name, "cancelled", {})    
+    #     if not for_restart:
+    #         store.log_run_event(context.ws, run_info.run_name, "cancelled", {})    
 
-        with run_info.lock:
-            run_info.killed_for_restart = for_restart
-            result, status, before_status = run_info.kill()
+    #     with run_info.lock:
+    #         run_info.killed_for_restart = for_restart
+    #         result, status, before_status = run_info.kill()
 
-            # try:
-            #     result, status, before_status = run_info.kill()
-            # except BaseException as ex:
-            #     logger.exception("Error in controller.cancel_core, ex={}".format(ex))
-            #     console.print("{}: got exception while trying to kill process; ex={}".format(run_info.run_name, ex))
-            #     # in controller code, always give up machine if we fail 
-            #     raise ex   
-
-        #console.print("run_info.kill returned result=", result, ", status=", status)
-        
-        # if run_info == self.single_run:
-        #     self.schedule_controller_exit()
-
-        return result, status, before_status
+    #     return result, status, before_status
 
     def get_matching_run_infos(self, full_run_names):
         # match all runinfos that have not finished (exact match and matching children)
@@ -1544,33 +1609,33 @@ class XTController(rpyc.Service):
         log_info("matches=", matches)
         return matches
 
-    def cancel_all(self, for_restart=False):
-        to_kill = []
-        results = []
+    # def cancel_all(self, for_restart=False):
+    #     to_kill = []
+    #     results = []
 
-        # loop until we are IDLE
-        while True:
-            with queue_lock:
-                to_kill += self.queue
-                self.queue = []
+    #     # loop until we are IDLE
+    #     while True:
+    #         with queue_lock:
+    #             to_kill += self.queue
+    #             self.queue = []
 
-            # grab all running jobs
-            with runs_lock:
-                running = [ri for ri in self.runs.values() if ri.status in ["running", "spawning"]] 
-                to_kill += running
+    #         # grab all running jobs
+    #         with runs_lock:
+    #             running = [ri for ri in self.runs.values() if ri.status in ["running", "spawning"]] 
+    #             to_kill += running
 
-            if not to_kill:
-                # all runs cancelled
-                break
+    #         if not to_kill:
+    #             # all runs cancelled
+    #             break
 
-            # kill jobs 1 at a time
-            for run_info in to_kill:
-                result, status, before_status = self.cancel_core(run_info, for_restart=for_restart)
+    #         # kill jobs 1 at a time
+    #         for run_info in to_kill:
+    #             result, status, before_status = self.cancel_core(run_info, for_restart=for_restart)
 
-                results.append( {"workspace": run_info.workspace, "run_name": run_info.run_name, 
-                    "cancelled": result, "status": status, "before_status": before_status} )
+    #             results.append( {"workspace": run_info.workspace, "run_name": run_info.run_name, 
+    #                 "cancelled": result, "status": status, "before_status": before_status} )
 
-        return results
+    #     return results
 
     def exposed_restart_controller(self, token, delay_secs=.01):
         self.validate_request(token)
@@ -1584,93 +1649,93 @@ class XTController(rpyc.Service):
 
         return True
 
-    def cancel_specified_runs(self, full_run_names):
-        to_kill = []
-        results = []
+    # def cancel_specified_runs(self, full_run_names):
+    #     to_kill = []
+    #     results = []
 
-        # loop until we are IDLE
-        while True:
+    #     # loop until we are IDLE
+    #     while True:
 
-            to_kill = self.get_matching_run_infos(full_run_names)
-            log_info("cancel_specified_runs: to_kill=", to_kill)
+    #         to_kill = self.get_matching_run_infos(full_run_names)
+    #         log_info("cancel_specified_runs: to_kill=", to_kill)
 
-            if not to_kill:
-                # we are IDLE
-                #console.print("----------- SPECIFIED RUNS ARE IDLE ---------------")
-                break
+    #         if not to_kill:
+    #             # we are IDLE
+    #             #console.print("----------- SPECIFIED RUNS ARE IDLE ---------------")
+    #             break
 
-            # kill jobs 1 at a time
-            while len(to_kill):
-                run_info = to_kill.pop(0)
-                result, status, before_status = self.cancel_core(run_info)
+    #         # kill jobs 1 at a time
+    #         while len(to_kill):
+    #             run_info = to_kill.pop(0)
+    #             result, status, before_status = self.cancel_core(run_info)
 
-                results.append( {"workspace": run_info.workspace, "run_name": run_info.run_name, 
-                    "cancelled": result, "status": status, "before_status": before_status} )
+    #             results.append( {"workspace": run_info.workspace, "run_name": run_info.run_name, 
+    #                 "cancelled": result, "status": status, "before_status": before_status} )
 
-        return results
+    #     return results
 
-    def cancel_runs_by_property_core(self, prop_name, prop_value):
-        to_kill = []
-        results = []
+    # def cancel_runs_by_property_core(self, prop_name, prop_value):
+    #     to_kill = []
+    #     results = []
 
-        # loop until we are IDLE
-        while True:
+    #     # loop until we are IDLE
+    #     while True:
 
-            to_kill = self.get_property_matching_run_infos(prop_name, prop_value)
-            log_info("cancel_runs_by_property_core: to_kill=", to_kill)
+    #         to_kill = self.get_property_matching_run_infos(prop_name, prop_value)
+    #         log_info("cancel_runs_by_property_core: to_kill=", to_kill)
 
-            if not to_kill:
-                # we are IDLE
-                #console.print("----------- SPECIFIED RUNS ARE IDLE ---------------")
-                break
+    #         if not to_kill:
+    #             # we are IDLE
+    #             #console.print("----------- SPECIFIED RUNS ARE IDLE ---------------")
+    #             break
 
-            # kill jobs 1 at a time
-            while len(to_kill):
-                run_info = to_kill.pop(0)
-                result, status, before_status = self.cancel_core(run_info)
+    #         # kill jobs 1 at a time
+    #         while len(to_kill):
+    #             run_info = to_kill.pop(0)
+    #             result, status, before_status = self.cancel_core(run_info)
 
-                results.append( {"workspace": run_info.workspace, "run_name": run_info.run_name, 
-                    "cancelled": result, "status": status, "before_status": before_status} )
+    #             results.append( {"workspace": run_info.workspace, "run_name": run_info.run_name, 
+    #                 "cancelled": result, "status": status, "before_status": before_status} )
 
-        return results
+    #     return results
 
     def exposed_shutdown(self, token):
         self.validate_request(token)
         print("shutdown request received...")
         self.schedule_controller_exit()
 
-    def exposed_cancel_run(self, token, run_names):
-        '''
-        run_names is a python list of ws/run-name entities.
-        '''
-        self.validate_request(token)
-        results = []
-        log_info("cancel_run: run_names=", run_names)
+    # def exposed_cancel_run(self, token, run_names):
+    #     '''
+    #     run_names is a python list of ws/run-name entities.
+    #     '''
+    #     self.validate_request(token)
+    #     results = []
+    #     log_info("cancel_run: run_names=", run_names)
 
-        if run_names == "all":
-            results = self.cancel_all()
-        else:
-            # kill specific run(s)
-            results = self.cancel_specified_runs(run_names)
+    #     if run_names == "all":
+    #         results = self.cancel_all()
+    #     else:
+    #         # kill specific run(s)
+    #         results = self.cancel_specified_runs(run_names)
 
-        # send results as json text so that client is not tied to controller (which may be killed immediately after this call)
-        results_json_text = json.dumps(results)
-        return results_json_text
+    #     # send results as json text so that client is not tied to controller (which may be killed immediately after this call)
+    #     results_json_text = json.dumps(results)
+    #     return results_json_text
 
-    def exposed_cancel_runs_by_property(self, token, prop_name, prop_value):
-        self.validate_request(token)
-        results = []
-        console.print("cancel_runs_by_property: prop_name=", prop_name, ", prop_value=", prop_value)
+    # def exposed_cancel_runs_by_property(self, token, prop_name, prop_value):
+    #     self.validate_request(token)
+    #     results = []
+    #     console.print("cancel_runs_by_property: prop_name=", prop_name, ", prop_value=", prop_value)
 
-        if prop_name == "username":
-            results = self.cancel_all()
-        else:
-            # kill specific run(s)
-            results = self.cancel_runs_by_property_core(prop_name, prop_value)
+    #     if prop_name == "username":
+    #         results = self.cancel_all()
+    #     else:
+    #         # kill specific run(s)
+    #         results = self.cancel_runs_by_property_core(prop_name, prop_value)
 
-        # send results as json text so that client is not tied to controller (which may be killed immediately after this call)
-        results_json_text = json.dumps(results)
-        return results_json_text      
+    #     # send results as json text so that client is not tied to controller (which may be killed immediately after this call)
+    #     results_json_text = json.dumps(results)
+    #     return results_json_text      
 
     def exposed_get_ip_addr(self, token):
         self.validate_request(token)
