@@ -55,8 +55,8 @@ def get_command_dicts(prev_exper, curr_exper, browse_flag, browse_opt, timeout_o
         },
 
 	    {
-            "title": "run script on Philly", 
-            "cmd": "xt run {} --target=philly --data-action=mount --model-action=download code/miniMnist.py --auto-download=0 --eval-model=1". \
-                format(monitor_opt), "needs_philly": True
+            "title": "run script on ITP-US", 
+            "cmd": "xt run {} --target=itp-us --data-action=mount --model-action=download code/miniMnist.py --auto-download=0 --eval-model=1". \
+                format(monitor_opt)
         },
 	]
\ No newline at end of file
