@@ -172,7 +172,7 @@ class XTConfig():
 
     def read_config(self, fn=None):
         if fn is None:
-            fn = file_utils.get_xtlib_dir() + "/helpers/default_config.yaml"
+            fn = file_utils.get_xtlib_dir() + "/helpers/" + constants.FACTORY_CONFIG
         self.fn = fn
 
         if not os.path.exists(fn):
@@ -180,7 +180,7 @@ class XTConfig():
             # if self.create_if_needed:
             #     console.print("XT config file not found; creating it from default settings...")
             #     file_dir = file_utils.get_xtlib_dir() + "/helpers"
-            #     from_fn = file_dir + "/default_config.yaml"
+            #     from_fn = file_dir + constants.FACTORY_CONFIG
 
             #     # read default file, update, write to fn
             #     text = file_utils.read_text_file(from_fn)
