@@ -14,6 +14,7 @@ from xtlib import utils
 from xtlib import constants
 from xtlib import run_helper
 from xtlib.hparams.hp_search_interface import HpSearchInterface
+from xtlib.console import console
 
 class BayesianSearch(implements(HpSearchInterface)):
     def __init__(self):
@@ -41,6 +42,13 @@ class BayesianSearch(implements(HpSearchInterface)):
 
             return trial
 
+        # print out the received set of hyperparameters to search
+        # for record in hp_records:
+        #     name_string = record["name"]
+        #     value_string = record["value"]
+        #     #console.print("hyperparameter: {} = {}".format(name_string, value_string))
+        #     console.print("hyperparameter: {}".format(record))
+
         dummy_loss = lambda x: None
         param_space = {r["name"]: r["space_func"] for r in hp_records} 
         domain = base.Domain(dummy_loss, param_space)
@@ -49,16 +57,31 @@ class BayesianSearch(implements(HpSearchInterface)):
 
         # convert runs to Trials
         trial_list = []
+        utils.log_info("len(runs)", len(runs))
 
         for run in runs:
             # don't trip over inappropriate runs
-            if (not "run_name" in run) or (not "hparams" in run) or (not context.primary_metric in run):
+            if not "run_name" in run:
+                console.print("run skipped: missing 'run_name' key")
                 continue
 
             run_name = run["run_name"]
+
+            if not "hparams" in run:
+                console.print("run '{}' skipped: missing 'hparams' key".format(run_name))
+                continue
+
+            if not "metrics" in run:
+                console.print("run '{}' skipped: missing 'metrics' key".format(run_name))
+                continue
+            
+            metrics = run["metrics"]
+            if not context.primary_metric in metrics:
+                continue
+
             arg_dict = run["hparams"]
 
-            loss_value = run[context.primary_metric]
+            loss_value = metrics[context.primary_metric]
             if context.maximize_metric:
                 loss_value = -loss_value
 
@@ -78,11 +101,15 @@ class BayesianSearch(implements(HpSearchInterface)):
 
         min_trials = 3      # before this, just do rand sampling
         seed =  rstate.randint(2 ** 31 - 1)
+        utils.log_info("len(trials)", len(trials))
+        utils.log_info("min_trials", min_trials)
 
         if len(trials) < min_trials:
+            utils.log_info("not enough trails", "using random HPs")
             new_trials = rand.suggest([tid], domain, trials, seed)
         else:
-            new_trials = tpe.suggest([tid], domain, trials, seed)
+            utils.log_info("sufficient trails", "using TPE suggested HPs")
+            new_trials = tpe.suggest([tid], domain, trials, seed, verbose=True)
 
         # apply the suggested hparam values
         trial = new_trials[0]
@@ -114,7 +141,7 @@ class BayesianSearch(implements(HpSearchInterface)):
                     if " " in value:
                         # surround with quotes so it is treated as a single entity
                         value = '"' + value + '"'
-            elif str(value.dtype) == "int32":
+            elif str(value.dtype) in ["int32", "int64"]:
                 value = int(value)
             elif hasattr(value, "real"):
                 value = float(value)
@@ -123,3 +150,6 @@ class BayesianSearch(implements(HpSearchInterface)):
 
         return hp_dict
 
+    def process_end_of_run(self, store, context, hp_records):
+        return False
+    
