@@ -26,7 +26,8 @@ from xtlib.hparams.hp_search_interface import HpSearchInterface
 # config - Short term for configuration.
 # RunSet - The class encapsulating a list of runs that share the same configuration.
 
-verbose = False
+verbose = True 
+verbose2 = False 
 
 dgd_rand = random.Random(time.time())  # For 'truly' random hyperparameter selection.
 # dgd_rand = random.Random(1)  # Deterministic, for debugging only!
@@ -44,13 +45,16 @@ class DGDSearch(implements(HpSearchInterface)):
         use dgd module to perform dgd search for next hparam set.
         '''
         dgd = DGD()
-        dgd.build(store, hp_records, runs, context.primary_metric)
+        dgd.build(store, hp_records, runs, context.primary_metric, maximize_metric=context.maximize_metric)
         
         runset = dgd.choose_config()
         arg_dict = dgd.arg_dict_from_runset(runset)
 
         return arg_dict
 
+    def process_end_of_run(self, store, context, hp_records):
+        return False
+
 
 class RunSet(object):
     def __init__(self, hp_id__setting_id__list, config_str):
@@ -68,7 +72,7 @@ class RunSet(object):
 
 
 class Run(object):
-    def __init__(self, run_summary, primary_metric):
+    def __init__(self, run_summary, primary_metric, maximize_metric=True):
         self.skip = False
         self.value = 0.
         self.run_name = run_summary["run_name"]
@@ -87,6 +91,10 @@ class Run(object):
             metrics = run_summary["metrics"]
             if primary_metric in metrics:
                 self.value = float(metrics[primary_metric])
+
+                if not maximize_metric:
+                    # flip the sign to search towards the minimum of primary metric
+                    self.value = -self.value
             else:
                 self.skip = True
                 if verbose:
@@ -155,15 +163,15 @@ class DGD(object):
         self.runsets = []
         self.configstr_runset_dict = {}
 
-    def build(self, store, records, all_runs, primary_metric, report=True):
+    def build(self, store, records, all_runs, primary_metric, report=True, maximize_metric=True):
         self.store = store
         start_time = time.time()
 
         self.define_hyperparameters(records)
-        self.create_run_objects(all_runs, primary_metric)
+        self.create_run_objects(all_runs, primary_metric, maximize_metric=maximize_metric)
         self.finalize_runsets()
 
-        if verbose:
+        if verbose2:
             console.print("All runsets...")
             for runset in self.runsets:
                 console.print('{}  {:4d} runs  {:10.3f}'.format(runset.config_str, len(runset.runs), runset.metric))
@@ -178,24 +186,19 @@ class DGD(object):
         for record in records:
             name_string = record["name"]
             value_string = record["value"]
-
-            # all our values are string but we only accept
-            # hparams with a list of values (containing commas)
-            if not "," in value_string:
-                console.print("skipping hparam with one value: {}={}".format(name_string, value_string))
-                continue
-
             console.print("hyperparameter: {} = {}".format(name_string, value_string))
-
             hp = Hyperparameter(len(self.hparams), name_string, value_string)
             self.hparams.append(hp)
             self.name_hparam_dict[name_string] = hp
 
-    def create_run_objects(self, run_reports, primary_metric):
+    def create_run_objects(self, run_reports, primary_metric, maximize_metric=True):
         self.runs = []
 
+        if verbose:
+            console.print("create_run_objects: len(run_reports)={}, primary_metric={}, maximize_metric={}".format(len(run_reports), primary_metric, maximize_metric))
+
         for record in run_reports:
-            run = Run(record, primary_metric)
+            run = Run(record, primary_metric, maximize_metric=maximize_metric)
             if run.skip:
                 continue
 
@@ -207,7 +210,7 @@ class DGD(object):
                     run.hpname_hpvalue_list.append((hp_name, run.hparams[hp_name]))
                 else:
                     run.skip = True  # Some hparam was not found in the report.
-                    if self.args.verbose:
+                    if verbose:
                         console.print("Skipping {}  (missing hparam={})".format(run.run_name, hp_name))
                     break
             if run.skip:
@@ -217,6 +220,11 @@ class DGD(object):
             hp_id__setting_id__list = []
             for hp_name, hp_value in run.hpname_hpvalue_list:
                 hparam = self.name_hparam_dict[hp_name]
+
+                # fixup multi-valued HP where we only use a single value
+                if isinstance(hp_value, list) and len(hp_value)==1:
+                    hp_value = hp_value[0]
+
                 if hp_value not in hparam.value_setting_dict:
                     if verbose:
                         console.print("Skipping {}  (Unexpected hparam value {}:{})".format(run.run_name, hparam.name, hp_value))
@@ -305,10 +313,15 @@ class DGD(object):
             probs[i] = ceiling - len(runset.runs)
         sum = np.sum(probs)
         probs /= sum
-        console.print("Runsets in the neighborhood of the best runset (at index 0)...")
-        for i, runset in enumerate(neighborhood):
-            runset.id = i
-            runset.report(" {:2d} prob={:6.4f}".format(runset.id, probs[i]))
+
+        console.print("Count of runsets in the neighborhood of the best runset (at index 0): {}".format(len(neighborhood)))
+
+        if verbose2:
+            console.print("Runsets in the neighborhood of the best runset (at index 0)...")
+            for i, runset in enumerate(neighborhood):
+                runset.id = i
+                runset.report(" {:2d} prob={:6.4f}".format(runset.id, probs[i]))
+
         chosen_runset = dgd_rand.choices(neighborhood, probs)[0]
         chosen_runset.report('\n {:2d} was chosen '.format(chosen_runset.id))
         return chosen_runset
@@ -338,6 +351,7 @@ class DGD(object):
 
         return arg_dict
 
+
 def fake_call_from_job(job_name, workspace, fn_hp_search):
     from xtlib.helpers.xt_config import get_merged_config
     config = get_merged_config()
@@ -361,7 +375,7 @@ def fake_call_from_job(job_name, workspace, fn_hp_search):
     hps = HParamSearch()
 
     search_file_text = store.read_job_file(context.ws, context.dest_name, context.hp_config)
-    arg_dict = hps.generate_hparam_set("fn_config", search_file_text, "run999", [], store, context)
+    arg_dict = hps.generate_hparam_set(search_file_text, "run999", store, context)
     print("{}".format(arg_dict))
 
 if __name__ == '__main__':
