@@ -155,10 +155,9 @@ class ImplHelp(ImplBase):
         console.print("    - command line options")
 
     def help_docs(self, browse=False):
-        url = "https://xtdocs.z22.web.core.windows.net/"
+        #url = "https://xtdocs.z22.web.core.windows.net/"
+        url = "https://xtdocsfordilbert.z22.web.core.windows.net/"
 
-        if self.basic_mode:
-            url = "https://txtdocs.z22.web.core.windows.net/"
 
         if browse:
             import webbrowser
