@@ -74,10 +74,14 @@ class StoreOnDemand(object):
             console.diag("compute_node detected, storage_creds=", storage_creds)
             console.diag("db_creds=", db_creds)
 
+            show_missing_jobid_calls = False
+
             # since we don't have a config file on the compute node, try to load our context from the MRC file
+
             context = utils.load_context_from_mrc()
             if context:
                 db_options = context.db_options
+                show_missing_jobid_calls = context.show_missing_jobid_calls
         else:
             # running on XT CLIENT machine; get them from config file
 
@@ -99,8 +103,10 @@ class StoreOnDemand(object):
             console.diag("db_creds=", db_creds)
             console.diag("xt client detected, storage_creds=", storage_creds)
 
+            show_missing_jobid_calls = self.config.get("logging", "show-missing-jobid-calls")
+
         self.store = Store(storage_creds, provider_code_path=provider_code_path, run_cache_dir=run_cache_dir, 
-            db_creds=db_creds, db_options=db_options)
+            db_creds=db_creds, db_options=db_options, show_missing_jobid_calls=show_missing_jobid_calls)
 
         console.diag("end of build_actual_store")
 
