@@ -394,7 +394,7 @@ class ImplUtilities(ImplBase):
 
     def get_config_template(self, template):
         # load default config file as lines
-        fn = file_utils.get_xtlib_dir() + "/helpers/default_config.yaml"
+        fn = file_utils.get_xtlib_dir() + "/helpers/" + constants.FACTORY_CONFIG
         default_text = file_utils.read_text_file(fn)
         default_lines = default_text.split("\n")
 
@@ -554,7 +554,7 @@ class ImplUtilities(ImplBase):
         '''
 
         if which == "factory":
-            fn = os.path.join(file_utils.get_xtlib_dir(), "helpers", "default_config.yaml")
+            fn = os.path.join(file_utils.get_xtlib_dir(), "helpers", constants.FACTORY_CONFIG)
             if not os.path.exists(fn):
                 errors.env_error("the XT default config file is missing: {}".format(fn))
         elif which == "global":
