@@ -21,11 +21,11 @@ from xtlib import run_helper
 job_info_props = {"_id": 1, "compute": 1, "concurrent": 1, "exper_name": 1, "hold": 1, "job_id": 1, "job_num": 1,
     "job_guid": 1, "job_secret": 1, "node_count": 1, "primary_metric": 1, "run_count": 1, "repeat": 1, 
     "schedule": 1, "search_type": 1, "search_style": 1, "username": 1, "xt_cmd": 1, "started": 1,
-    "pool_info": 1, "service_job_info": 1, "ws_name": 1}
+    "pool_info": 1, "service_job_info": 1, "ws_name": 1, "sleep_on_exit": 1}
 
 job_stats_props = {"job_status": 1, "completed_runs": 1, "error_runs": 1, "running_runs": 1, 
     "running_nodes": 1, "total_runs": 1, "db_retries": 1, "storage_retries": 1,
-    "restarts": 1}
+    "restarts": 1, "next_run_index": 1}
 
 def is_job_id(name):
     # compatible with version 1 (prefix-jobNNNN) and version 2 (jobNNNN)
@@ -163,18 +163,18 @@ def build_job_filter_dict(ws_name, job_list, user_to_actual, builder, workspace,
 
 user_to_actual = {"cluster": "pool_info.cluster", "docker": "pool_info.environment", "experiment": "exper_name",
     "hold": "hold", "job": "job_id","job_num": "job_num", "low_pri": "pool_info.low-pri", 
-    "nodes": "pool_info.nodes", "primary_metric": "primary_metric", "queue": "pool_info.queue", 
+    "node_count": "node_count", "primary_metric": "primary_metric", "queue": "pool_info.queue", 
     "repeat": "repeat", "search": "search_type", "service_type": "pool_info.service", 
     "sku": "pool_info.sku", "runs": "run_count", "search_style": "search_style", "started": "started", "target": "compute", "username": "username", 
     "vc": "pool_info.vc", "vm_size": "pool_info.vm-size", "workspace": "ws_name",  
-    "hparams": "hparams", "tags": "tags", 
+    "hparams": "hparams", "tags": "tags", "sleep_on_exit": "sleep_on_exit", 
 
 
     # dynamic properties (get updated at various stages of job)
     "status": "job_status", "running_nodes": "running_nodes", 
     "running_runs": "running_runs", "error_runs": "error_runs", "completed_runs": "completed_runs",
     "total_runs": "total_runs", "db_retries": "db_retries", "storage_retries": "storage_retries",
-    "restarts": "restarts"
+    "restarts": "restarts", "next_run_index": "next_run_index",
     }
 
 std_cols_desc = {
@@ -186,7 +186,7 @@ std_cols_desc = {
     "job": "the job name",
     "job_num": "the sort compatible, numeric portion of the job_id",
     "low_pri": "if the nodes requested for this job were specified as preemptable",
-    "nodes": "the number of compute nodes requested for this job",
+    "node_count": "the number of compute nodes requested for this job",
     "primary_metric": "the name of the metric used for hyperparameter searching",
     "queue": "the Philly queue that the job was submitted to",
     "repeat": "the repeat count specified for the job",
@@ -195,6 +195,7 @@ std_cols_desc = {
     "search_style": "describes how search is being accomplished (one of: static, dynamic, multi, repeat, single)",
     "service_type": "the service type of the target",
     "sku": "the machine type requested for this job",
+    "sleep_on_exit": "the amount of time controller should wait before it's normal process exist",
     "started": "the datetime when the job was started",
     "target": "the name of the compute target the job was submitted to",
     "username": "the login name of the user who submitted the job",
@@ -211,7 +212,8 @@ std_cols_desc = {
     "error_runs": "the number of runs that have terminated with an error",
     "storage_retries": "the total number of storage retries performed by the runs of this node",
     "total_runs": "the total number of runs specified for the job",
-    "restarts": "the total number node restarts in this job",
+    "restarts": "the number of times a node has been preempted and restarted",
+    "next_run_index": "the number to be used for the next dynamic run within the job"
 }
 
 def get_job_property_dicts():
@@ -223,7 +225,7 @@ def get_job_property_internal_name(name):
         return user_to_actual[name]
     return None
 
-def get_list_jobs_records(store, config, args):
+def get_list_jobs_records(store, config, user_columns, args):
     list_nodes = utils.safe_value(args, "list_nodes")
 
     job_list = args["node_list"] if list_nodes else args["job_list"]
@@ -260,8 +262,10 @@ def get_list_jobs_records(store, config, args):
 
     # get the db records for the matching JOBS
     #console.print("gathering job data...", flush=True)
-    records, using_default_last, last = builder.get_db_records(db, filter_dict, workspace, "jobs", actual_to_user, args=args)
-    return records, using_default_last, last, user_to_actual, builder
+    records, limiter, limiter_value = builder.get_db_records(db, filter_dict, workspace, "jobs", actual_to_user, 
+        user_columns=user_columns, args=args)
+
+    return records, limiter, limiter_value, user_to_actual, builder
 
 
 def list_jobs(store, config, args, compare=False):
@@ -269,12 +273,14 @@ def list_jobs(store, config, args, compare=False):
     console.diag("job_helper.list_jobs")
 
     available = args["available"]
+    report_type = "job-reports"
+    user_columns = utils.get_user_columns(args["columns"], report_type, config)
 
     if compare:
         args["columns"] = ["job", "hparams.*"]
 
     records, using_default_last, last, user_to_actual, builder \
-        = get_list_jobs_records(store, config, args)
+        = get_list_jobs_records(store, config, user_columns=user_columns, args=args)
 
     console.diag("after get_list_jobs_records")
 
@@ -295,7 +301,7 @@ def list_jobs(store, config, args, compare=False):
             args["columns"] = diff_cols 
 
         avail_list = list(user_to_actual.keys())
-        lines, row_count, was_exported = builder.build_report(records, report_type="job-reports", args=args)
+        lines, row_count, was_exported = builder.build_report(records, user_columns=user_columns, report_type=report_type, args=args)
 
         store_name = config.get("store")
         workspace = args["workspace"]
@@ -316,7 +322,7 @@ def list_jobs(store, config, args, compare=False):
 
                 if row_count > 1:
                     if using_default_last:
-                        console.print("total jobs listed: {} (defaulted to --last={})".format(row_count, last))
+                        console.print("total jobs listed: {} (limited by --last={})".format(row_count, last))
                     else:
                         console.print("total jobs listed: {}".format(row_count))
             else:
