@@ -5,7 +5,7 @@
 # self.py: functions to help produce plots of metrics from runs
 import math
 import time
-from azureml.core import experiment
+from tokenize import group
 import numpy as np
 import pandas as pd
 
@@ -17,6 +17,7 @@ from xtlib import constants
 from xtlib import run_helper
 from xtlib import job_helper
 from xtlib import store_utils
+from xtlib.hp_set import HpSetFormatter
 
 PRESMOOTH = "_PRE-SMOOTH_"
 ERR = "_ERR_"
@@ -33,7 +34,7 @@ class PlotBuilder():
             x_min, x_max, y_min, y_max, alpha, color_indexes, line_styles, line_sizes, timebase, x_format, y_format, 
             x_share, y_share, background_color, grid_color, clean, width, height, highlight, color_highlight,
             legend_suffix, col_source, x_int, y_int, bins, legend_vars, margins, metric_aliases, y_ticks,
-            plot_args):
+            first, last, skip, plot_args):
         
         self.store = store
         self.group_names = None
@@ -103,6 +104,10 @@ class PlotBuilder():
         self.metric_aliases = metric_aliases
         self.all_visible = True
         self.y_ticks = y_ticks
+        self.first = first
+        self.last = last
+        self.skip = skip
+        self.hp_set_formatter = HpSetFormatter()
 
         if colors:
             self.colors = colors
@@ -131,6 +136,9 @@ class PlotBuilder():
 
         group_order_dict = {}
 
+        # from the current set of records, calculate the set of hyperparameters that change between runs
+        self.hp_set_formatter.build_hp_set_names(self.run_log_records)
+
         data_frames_by_cols = self.build_data_frames(group_order_dict)
         if data_frames_by_cols:
 
@@ -147,16 +155,24 @@ class PlotBuilder():
 
         if group_by == "run":
             value = run
+
         elif group_by == "node":
             value = node
+
         elif group_by == "job":
             value = job
+
         elif group_by == "experiment":
             value = experiment
+
         elif group_by == "workspace":
             value = experiment
+
         elif group_by in record:
             value = record[group_by]
+            if group_by == "hp_set":
+                value = self.hp_set_formatter.format_hpset_simple(value)
+
         else:
             raise Exception("unrecognized group-by name: " + str(group_by))
 
@@ -199,7 +215,11 @@ class PlotBuilder():
             if not group_value in group_order_dict:
                 group_order_dict[group_value] = 1
 
-            info_record = {"run": run, "job": job, "node": node, "experiment": experiment}
+            info_record = {"run": run, "job": job, "node": node, "experiment": experiment}   
+
+            # add group_value
+            if self.group_by and not self.group_by in info_record:
+                info_record[self.group_by] = group_value
 
             # fixup any aggregated items as "*"
             if not self.group_by in ["run"]:
@@ -419,6 +439,15 @@ class PlotBuilder():
             df["experiment"] = [experiment] * count
             df["workspace"] = [workspace] * count
 
+            # add group_by 
+            if self.group_by and not self.group_by in df:
+                group_value = record[self.group_by]
+
+                if self.group_by == "hp_set":
+                    group_value = self.hp_set_formatter.format_hpset_simple(group_value)
+
+                df[self.group_by] = [group_value] * count
+
             # add legend title variables
             for var in self.legend_title_vars:
                 df[var] = record[var]
@@ -427,7 +456,8 @@ class PlotBuilder():
                 data_frames_by_cols[cols] = df
             else:
                 dfx = data_frames_by_cols[cols]
-                dfx = dfx.append(df)
+                #dfx = dfx.append(df)
+                dfx = pd.concat([dfx, df])
                 data_frames_by_cols[cols] = dfx
 
         return missing_col_runs
@@ -620,7 +650,7 @@ class PlotBuilder():
 
                 record_count = len(df.index)
 
-                if record_count:
+                if record_count:  #  or force_value:
                     if force_floats:
                         y_values = make_floats(df[ y_col ])
                     else:
@@ -865,7 +895,9 @@ class PlotBuilder():
         elif plot_rows > 1:
             plots = plots.flatten()
 
-        fig.suptitle(self.title, fontsize=16)
+        group = self.group_names[0]
+        title = self.fixup_text(self.title, group, None)
+        fig.suptitle(title, fontsize=16)
 
         if self.timeout:
             # build a thread to close our plot window after specified time
@@ -987,12 +1019,13 @@ class PlotBuilder():
 
         for ax in plots:
             legend = ax.get_legend()
-            legend_lines = legend.get_lines()
+            if legend:
+                legend_lines = legend.get_lines()
 
-            for i, legend_line in enumerate(legend_lines):
-                legend_line.set_picker(True)  # 5 pts tolerance
-                legend_line.set_pickradius(5)
-                plot_dict[legend_line] = i
+                for i, legend_line in enumerate(legend_lines):
+                    legend_line.set_picker(True)  # 5 pts tolerance
+                    legend_line.set_pickradius(5)
+                    plot_dict[legend_line] = i
 
         def onpick(event):
             # on the pick event, find the orig line corresponding to the
@@ -1086,14 +1119,14 @@ class PlotBuilder():
                     else:
                         texts.set_fontproperties(normal_font)
 
-    def get_alive_status(self, names):
+    def get_alive_status(self, group_values):
         alive_dict = {}
-        first_name = names[0]
 
-        if run_helper.is_run_name(first_name):
+        #if run_helper.is_run_name(first_name):
+        if self.group_by == "run":
 
             # are specified RUNS alive?
-            id_list = [store_utils.make_id(self.ws_name, name) for name in names]
+            id_list = [store_utils.make_id(self.ws_name, name) for name in group_values]
             filter_dict = {"_id": {"$in": id_list}}
             fields_dict = {"_id": 1, "status": 1}
             
@@ -1106,9 +1139,10 @@ class PlotBuilder():
                     run_name = record["_id"]
                     alive_dict[run_name] = True
 
-        elif job_helper.is_job_id(first_name):
+        #elif job_helper.is_job_id(first_name):
+        elif self.group_by == "job":
             # are RUNS alive in specified JOBS
-            filter_dict = {"job_id": {"$in": names}}
+            filter_dict = {"job_id": {"$in": group_values}}
             fields_dict = {"job_id": 1, "status": 1}
             
             records = self.store.database.get_info_for_runs(self.ws_name, filter_dict, fields_dict)
@@ -1120,9 +1154,9 @@ class PlotBuilder():
                     job_name = record["job_id"]
                     alive_dict[job_name] = True
 
-        else:
+        elif self.group_by == "experiment":
             # are RUNS alive in specified EXPERIMENTS
-            filter_dict = {"exper_name": {"$in": names}}
+            filter_dict = {"exper_name": {"$in": group_values}}
             fields_dict = {"exper_name": 1, "status": 1}
             
             records = self.store.database.get_info_for_runs(self.ws_name, filter_dict, fields_dict)
@@ -1134,6 +1168,10 @@ class PlotBuilder():
                     exper_name = record["exper_name"]
                     alive_dict[exper_name] = True
 
+        else:
+            # for now, don't support alive for custom group by (e.g., hp_set)
+            pass
+
         return alive_dict
 
     def get_seaborn_color_map(self, name, n_colors=5):
@@ -1230,6 +1268,9 @@ class PlotBuilder():
                 line_title += self.legend_suffix
 
             line_title = self.fixup_text(line_title, group_name, col)
+            if line_title == "-":
+                # hide series with user-specified blank titles
+                return
 
         else:
             line_title = None
@@ -1330,52 +1371,56 @@ class PlotBuilder():
             ax.set_xticklabels(x_labels)
 
     def fixup_text(self, text, group_name, col):
-        info = self.group_info[group_name]
+        if text:
+            group_info = self.group_info[group_name] if group_name else None
 
-        if "$job" in text:
-            job_name = info["job"]
-            text = text.replace("$job", job_name)
+            if "$job" in text:
+                job_name = group_info["job"]
+                text = text.replace("$job", job_name)
 
-        if "$run" in text:
-            run_name = info["run"]
-            text = text.replace("$run", run_name)
+            if "$run" in text:
+                run_name = group_info["run"]
+                text = text.replace("$run", run_name)
 
-        if "$node" in text:
-            node_name = info["node"]
-            text = text.replace("$node", node_name)
-        
-        if "$experiment" in text:
-            exper_name = info["experiment"]
-            text = text.replace("$experiment", exper_name)
-        
-        text = text.replace("$group", group_name if group_name else "")
-        text = text.replace("$col", col)
+            if "$node" in text:
+                node_name = group_info["node"]
+                text = text.replace("$node", node_name)
+            
+            if "$experiment" in text:
+                exper_name = group_info["experiment"]
+                text = text.replace("$experiment", exper_name)
+            
+            if "$group" in text:
+                text = text.replace("$group", group_name if group_name else "")
 
-        if "@" in text:
-            # look for a named column (@hparams.foo@) and replace it with its value
-            parts = text.split("@")
-            if len(parts) > 2:
-                # contains 1 or more @xxx@ entries
-                new_text = ""
+            if "$col" in text:
+                text = text.replace("$col", col)
 
-                while len(parts) >= 2:
-                    left = parts.pop(0)
-                    var_index = parts.pop(0)
-                    col_name = self.legend_title_vars[int(var_index)]
-                    
-                    # update @N@ with value of arg   
-                    if col_name in info:
-                        value = info[col_name]
-                    else:
-                        _, value, _ = self.get_xy_values(self.data_frames_by_cols, group_name, x_col=None, y_col=col_name, 
-                            stat_col=None, force_floats=False)
+            if "@" in text:
+                # look for a named column (@hparams.foo@) and replace it with its value
+                parts = text.split("@")
+                if len(parts) > 2:
+                    # contains 1 or more @xxx@ entries
+                    new_text = ""
 
-                        if isinstance(value, (tuple, list)):
-                            value = value[0]
-                    new_text += "{}{}".format(left, value)
+                    while len(parts) >= 2:
+                        left = parts.pop(0)
+                        var_index = parts.pop(0)
+                        col_name = self.legend_title_vars[int(var_index)]
+                        
+                        # update @N@ with value of arg   
+                        if group_info and col_name in group_info:
+                            value = group_info[col_name]
+                        else:
+                            _, value, _ = self.get_xy_values(self.data_frames_by_cols, group_name, x_col=None, y_col=col_name, 
+                                stat_col=None, force_floats=False, force_value=True)
 
-                new_text += parts.pop()
-                text = new_text
+                            if isinstance(value, (tuple, list)):
+                                value = value[0]
+                        new_text += "{}{}".format(left, value)
+
+                    new_text += parts.pop()
+                    text = new_text
 
         return text
 
