@@ -3,6 +3,7 @@
 # Licensed under the MIT license.
 #
 # qfe: a quick-front-end builder for XT (parses commands, arguments, and options & dispatches to associated code).
+import enum
 import os
 import sys
 import copy
@@ -15,6 +16,7 @@ from xtlib import utils
 from xtlib import errors
 from xtlib import run_helper
 from xtlib import job_helper
+from xtlib import qfe
 
 '''
 Definitions:
@@ -1162,11 +1164,13 @@ class Dispatcher():
 
     def replace_curlies_with_quotes(self, text):
         '''
-        replace any {} that appear outside of quotes with single quotes.
+        replace any {} that appear outside of quotes with single quotes UNLESS
+        they are part of a --template option value.
         '''
 
         new_text = ""
         protector = None
+        inside_template = False
 
         for ch in text:
             if ch == protector:
@@ -1190,6 +1194,35 @@ class Dispatcher():
 
         return new_text
 
+    def extract_template_option(self, args):
+        template_option = None
+        new_args = []
+        parsing_template = False
+
+        for arg in args:
+
+            if arg.startswith("--"):
+                # start of an option
+                left = arg.split("=")[0]
+                opt_name = left[2:].strip()
+
+                if match(opt_name, "template"):
+                    parsing_template = True
+                    template_option = arg
+                else:
+                    parsing_template = False
+                    new_args.append(arg)
+
+            elif parsing_template:
+                # another value of template
+                template_option += " " + arg
+
+            else:
+                new_args.append(arg)
+
+
+        return template_option, new_args
+        
     def dispatch(self, args, is_rerun=False, capture_output=False, raise_syntax_exception=False):
         self.raise_syntax_exception = raise_syntax_exception
         
@@ -1197,12 +1230,24 @@ class Dispatcher():
         global explict_options
         explict_options = {}
 
-        orig_text = " ".join(args)
-        self.dispatch_cmd = orig_text
+        # extract the template option since it has curly brackets that are not converted to quotes
+        template_option, args = self.extract_template_option(args)
+
+        # join args together to get original cmd line (but without quotes)
+        text = " ".join(args)
+
+        # NOTE: Windows/Linux command line parsers typically strip away quotes found in cmd
+        # XT allows allows the user to use {} as a 2nd set of quotes around complex values (that won't get stripped away)
+        text = self.replace_curlies_with_quotes(text)
+
+        # add back template option
+        if template_option:
+            text += " " + template_option
 
-        text = self.replace_curlies_with_quotes(orig_text)
         console.diag("\nADJUSTED cmd={}".format(text))
 
+        self.dispatch_cmd = text
+
         scanner = Scanner(text)
         tok = scanner.scan()
         #console.print("first tok=", tok)
@@ -1449,8 +1494,14 @@ class Dispatcher():
                 value = self.parse_flag_value(name, value)
             else:
                 value = True
+
+        elif opt_type == "template_value":
+            # get rest of str (any chars except a space)
+            value = tok + scanner.get_rest_of_token_until_space()
+            tok = scanner.scan()
+
         else:
-            # its a simple value
+            # its a simple value type
             value = tok
             tok = scanner.scan()
 
@@ -1469,6 +1520,7 @@ class Dispatcher():
                 else:
                     self.syntax_error("Illegal value for boolean option: " + str(value))
             elif opt_type == "str":
+                #value = scanner.get_value_until_space()
                 value = self.expand_system_values(value)
 
             else:
