@@ -21,12 +21,14 @@ from xtlib import utils
 from xtlib import errors
 from xtlib import pc_utils
 from xtlib import constants
+from xtlib.hp_set import HpSetFormatter
 
 class ReportBuilder():
     def __init__(self, config=None, store=None):
         self.config = config
         self.store = store
         self.user_col_args = {}
+        self.hp_set_formatter = HpSetFormatter()
 
     def wildcard_matches_in_list(self, wc_name, name_list, omits):
         matches = []
@@ -203,13 +205,13 @@ class ReportBuilder():
             else:
                 col_dict[prefix + key] = 1
 
-    def flatten_records(self, records, sort_col, need_alive, which, args):
+    def flatten_records(self, records, sort_col, need_alive, which, user_columns=None, args=None):
         '''
         pull out the AVAILABLE or USER SPECIFIED columns, flattening nested props to their dotted names.
         '''
 
         # build avail col list based on final set of filtered records
-        avail_cols, actual_cols, user_col_args  = self.get_avail_actual_user_cols(records, args)
+        avail_cols, actual_cols, user_col_args  = self.get_avail_actual_user_cols(records, user_columns, args)
 
         # add often-needed cols for processing report
         for col in ["status", "created", "started"]:
@@ -301,7 +303,8 @@ class ReportBuilder():
         return first, last
 
     def get_db_records(self, database, filter_dict, workspace, which, actual_to_user,
-            col_dict=None, col_names_are_external=True, flatten_records=True, need_alive=False, args=None):
+            col_dict=None, col_names_are_external=True, flatten_records=True, need_alive=False, 
+            user_columns=None, args=None):
 
         started = time.time()
 
@@ -312,16 +315,48 @@ class ReportBuilder():
         if list_nodes:
             which = "nodes"
 
-        if last:
-            using_default_last = True
+        if first:
+            limiter = "first"
+            limiter_value = first
+        elif last:
+            limiter = "last"
+            limiter_value = last
         else:
-            using_default_last = False
+            limiter = None
+            limiter_value = 0
 
         reverse = utils.safe_value(args, "reverse")
         # use database to do all of the work (query, sort, first/last)
-        sort_col = utils.safe_value(args, "sort", "name")
+        sort_col = utils.safe_value(args, "sort")   
+        group_col = utils.safe_value(args, "group")   
         sort_dir = 1
 
+        # group vs. sort
+        if group_col:
+            def _get_name(text): 
+                if "=" in text:
+                    text = text.split("=", 1)[0]
+                if ":" in text:
+                    text = text.split(":", 1)[0]
+                return text
+
+            user_col_names = [_get_name(col_format) for col_format in user_columns]
+            if not group_col in user_col_names:
+                user_columns.append(group_col)
+
+            # if sort_col and group_col != sort_col:
+            #     raise Exception("cannot specify both --sort and --group")
+            # sort by grouping col
+            sort_col = group_col
+
+            # zap first/last/reverse flags for grouping phase
+            first = None
+            last = None
+            reverse = False
+
+        elif not sort_col:
+            sort_col = "name"
+        
         if sort_col:
             if sort_col == "name":
 
@@ -396,7 +431,7 @@ class ReportBuilder():
 
         if flatten_records:
             # pull out requested cols, flattening nested values to their dotted names
-            records = self.flatten_records(records, sort_col, need_alive, which, args)
+            records = self.flatten_records(records, sort_col, need_alive, which, user_columns=user_columns, args=args)
 
         if last:
             # we had to reverse the sort done by database, so correct it here
@@ -406,17 +441,19 @@ class ReportBuilder():
         elapsed = time.time() - started
         #console.print("  query stats: round trips={}, elapsed: {:.2f} secs".format(round_trip_count, elapsed))
 
-        return records, using_default_last, last
+        return records, limiter, limiter_value
 
-    def get_user_columns(self, args):
-        requested_list = args["columns"]
+    def get_user_columns(self, requested_list, args):
+        if requested_list is None:
+            requested_list = args["columns"]
+            
         add_cols = utils.safe_value(args, "add_columns")
         if add_cols:
             requested_list += add_cols
 
         return requested_list
 
-    def get_avail_actual_user_cols(self, records, args):
+    def get_avail_actual_user_cols(self, records, user_columns, args):
         col_dict = OrderedDict()
         for sr in records:
             if "metric_names" in sr:
@@ -432,15 +469,15 @@ class ReportBuilder():
         avail_list = list(col_dict.keys())
 
         # get list of user-requested columns
-        user_columns = self.get_user_columns(args)
+        all_user_columns = self.get_user_columns(user_columns, args)
 
         # parse out the custom column NAMES and FORMATS provided by the user
-        user_col_args = self.build_user_col_args(user_columns)
+        user_col_args = self.build_user_col_args(all_user_columns)
 
         actual_cols, user_col_args = self.get_requested_cols(user_col_args, avail_list)
         return avail_list, actual_cols, user_col_args
 
-    def build_report(self, records, report_type, args):
+    def build_report(self, records, user_columns, report_type, args):
         ''' build a tabular report of the records, or export to a table, as per args.  
         values in each record must have been flattened with a dotted key (e.g., hparams.lr).
         records must be in final sort order.
@@ -449,7 +486,57 @@ class ReportBuilder():
         was_exported = False
         max_column_width = utils.safe_value(args, "max_width")
 
-        avail_list, actual_cols, user_col_args  = self.get_avail_actual_user_cols(records, args)
+        hp_set_display = utils.safe_value(args, "hp_set")
+        mean_cols = utils.safe_value(args, "mean")
+        group_by = utils.safe_value(args, "group")
+        sort_by = utils.safe_value(args, "sort")
+        reverse = utils.safe_value(args, "reverse")
+        first = utils.safe_value(args, "first")
+        last = utils.safe_value(args, "last")
+
+        if mean_cols:
+            # produce new mean-calculated columns (new records)
+            records = self.add_dynamic_cols(records, user_columns)
+            records = self.create_mean_records(user_columns, records, mean_cols, group_by, sort_by, reverse, first, last)
+            if args["flat"]:
+                group_by = None
+
+        # from the current set of records, calculate the set of hyperparameters that change between runs
+        self.hp_set_formatter.build_hp_set_names(records)
+
+        if True:       # hp_set_display == "hidden":
+            # remove all hp_set column/format info
+            cols = list(user_columns)
+            for col in cols:
+                if col == "hp_set" or col.startswith("hp_set=") or col.startswith("hp_set:"):
+                    user_columns.remove(col)
+
+        if hp_set_display == "changed":
+            user_columns.append("hp_set:$hpset_changed")
+
+        if hp_set_display == "simple":
+            user_columns.append("hp_set:$hpset_simple")
+
+        elif hp_set_display == "full":
+            user_columns.append("hp_set")
+
+        elif hp_set_display == "columns":
+            if self.hp_set_formatter.hp_common_set:
+                # for each run record, convert hp_set column into new columns in the record 
+                for record in records:
+                    hp_set_str = record["hp_set"]
+                    hp_set = self.hp_set_formatter.format_hpset_changed(hp_set_str)
+
+                    for hp, value in hp_set.items():
+                        full_hp_name = "hparams." + hp
+                        record[full_hp_name] = value
+
+                    del record["hp_set"]
+
+                # add "hparams.*" to ensure we display all of the hyperparameters
+                #user_columns.append("hparams.*")
+
+        avail_list, actual_cols, user_col_args  = self.get_avail_actual_user_cols(records, user_columns, args)
 
         # store for later use
         self.user_col_args = user_col_args
@@ -467,10 +554,9 @@ class ReportBuilder():
             count = self.export_records(fn_export, records, col_list, sep_char)
             row_count = count - 1
             line = "report exported to: {} ({} rows)".format(fn_export, row_count)
-            lines = [line]
+            lines += [line]
             was_exported = True
         else:
-            group_by = args["group"] if "group" in args else None
             number_groups = args["number_groups"] if "number_groups" in args else False
             actual_cols = list(user_col_args.keys())
 
@@ -526,11 +612,7 @@ class ReportBuilder():
         for col_spec in requested_list:
             col_name = col_spec
             user_fmt = None
-
-            if "." in col_name:
-                prefix, col_name = col_name.split(".", 1)
-            else:
-                prefix = None
+            user_name = None
 
             if "=" in col_name:
                 # CUSTOM NAME
@@ -545,6 +627,14 @@ class ReportBuilder():
                     col_name, user_fmt = col_name.split(":")
                     
                     user_fmt = "{:" + user_fmt + "}"
+
+            # only look for "." after we have isolated the actual col_name (from the formatting info)
+            if "." in col_name:
+                prefix, col_name = col_name.split(".", 1)
+            else:
+                prefix = None
+
+            if not user_name:
                 user_name = col_name
 
             # rebuild prefix_name (must be prefix + col_name)
@@ -558,26 +648,40 @@ class ReportBuilder():
         blank_zero_fmt = "{:$bz}"
         date_only = "{:$do}"
         time_only = "{:$to}"
+        hpset_simple = "{:$hpset_simple}"
+        hpset_changed = "{:$hpset_changed}"
         right_trim = "{:$rt."
+        align = ">"
 
         if fmt == blank_zero_fmt:
             # blank if zero
             value = "" if value == 0 else str(value)
+
         elif fmt == date_only:
             # extract date portion of datetime string
             delim = " " if " " in value else "T"
             value, _ = value.split(delim)
+
         elif fmt == time_only:
             # extract time portion of datetime string
             delim = " " if " " in value else "T"
             _, value = value.split(delim)
             value = value.split(".")[0]    # extract hh:mm:ss part of fractional time
+
+        elif fmt == hpset_simple:
+            value = str(self.hp_set_formatter.format_hpset_simple(value))
+            align = "<"
+
+        elif fmt == hpset_changed:
+            value = str(self.hp_set_formatter.format_hpset_changed(value))
+            align = "<"
+
         elif fmt.startswith(right_trim):
             int_part = fmt[6:].split("}")[0]
             trim_len = int(int_part)
             value = str(value)[-trim_len:]
         
-        return value
+        return value, align
 
     def needed_precision(self, value, significance):
         # start with scientific notation formatting
@@ -590,7 +694,13 @@ class ReportBuilder():
         sd, exponent = text.split("e")
         exponent = int(exponent)
 
-        needed = max(0, significance - (exponent+1))
+        # needed = max(0, significance - (exponent+1))
+        if exponent <= 0:
+            #my_precision = 1 - exponent
+            needed = 2 - exponent
+        else:
+            needed = 0
+
         return needed
 
     def smart_float_format(self, value, significance, max_precision, max_fixed_length):
@@ -605,13 +715,20 @@ class ReportBuilder():
     def build_formatted_table(self, records, avail_cols=None, col_list=None, max_col_width=None, 
         report_type="run-reports", group_by=None, number_groups=False):
         '''
-        builds a nicely formatted text table from a set of records.
+        Args:
+            records:        a list of dict entries containing data to format
+            avail_cols:     list of columns (unique dict keys found in records)
+            col_list:    list of columns to be used for report (strict subset of 'avail_cols')
+            report_type:    a dotted path to the report definition in the XT config file
 
-        'records' - a list of dict entries containing data to format
-        'avail_cols' - list of columns (unique dict keys found in records)
-        'actual_cols' - list of columns to be used for report (strict subset of 'avail_cols')
+        Processing:
+            Builds a nicely formatted text table from a set of records.
         '''
 
+        if group_by and group_by in col_list:
+            # remove group_by columns from those display in group records
+            col_list.remove(group_by)
+
         time_col_names = ["created", "started", "ended", "time"]
         duration_col_names = ["duration", "queued", "queue_duration", "prep_duration",
             "app_duration", "post_duration"]
@@ -684,13 +801,16 @@ class ReportBuilder():
                 if not col in record:
                     # not all columns are defined in all records
                     continue
-
                 try:
                     value = record[col]
 
+                    if col == "hp_set":
+                        dummy = 0
+
                     # special formatting for time values
                     if col in duration_col_names:
                         value = self.format_duration(value, col, record)
+
                     elif col in time_col_names:
                         if isinstance(value, str):
                             value = arrow.get(value)
@@ -699,15 +819,23 @@ class ReportBuilder():
                     # for sql, convert float types that are really integer values to int types
                     if isinstance(value, float) and value == int(value) and user_fmt is None and value_type is str:
                         value = int(value)
+                    # elif isinstance(value, float) and value_type is int:
+                    #     value = int(value)
+
+                    elif isinstance(value, str) and value.isnumeric():
+                        # handle cases where int was logged incorrectly as a string
+                        value = int(value)
 
                     if user_fmt:
                         # user provided explict format for this column
                         if "$" in user_fmt:
-                            value_str = self.xt_custom_format(user_fmt, value)
+                            value_str, align = self.xt_custom_format(user_fmt, value)
                         else:
                             value_str = user_fmt.format(value)
-                    elif isinstance(value, float):
 
+                    elif isinstance(value, float):
+                        if col == "metrics.best-eval-acc":
+                            dummy = 33
                         # default (smart) FLOAT formatting
                         needed = self.needed_precision(value, significance)
                         max_precision = max(max_precision, needed)
@@ -721,11 +849,13 @@ class ReportBuilder():
                         value_str = str(value)
                         value_type = bool
                         is_numeric = False
+
                     elif isinstance(value, int):
                         value_str = int_fmt.format(value)
                         if value_type == str:
                             value_type = int
                             is_numeric = True
+
                     elif value is not None:
                         # don't let None values influence the type of field
                         # assume value found is string-like
@@ -736,6 +866,7 @@ class ReportBuilder():
                         value_str = str_fmt.format(value) if value else ""
                         if first_value:
                             is_numeric = utils.str_is_float(value)
+
                     else:
                         value_str = ""
 
@@ -746,11 +877,18 @@ class ReportBuilder():
                     console.print("Exception formatting col={}: {}".format(col, ex))
                     console.print("  Exception record: {}".format(record))
 
+                    # debug
+                    raise ex
+
             # finish this col
             if is_numeric and not max_precision:
                 max_precision = 3
 
-            col_width = min(max_col_width, col_width)
+            if col == "hp_set":
+                dummy = col
+                
+            if not user_fmt:
+                col_width = min(max_col_width, col_width)
 
             col_info = {"name": col, "user_name": user_col, "col_width": col_width, "value_type": value_type, 
                 "is_numeric": is_numeric, "precision": max_precision, "user_fmt": user_fmt, 
@@ -771,7 +909,7 @@ class ReportBuilder():
                 if number_groups:
                     text += "\n{}. {}:\n".format(i+1, group)
                 else:
-                    text += "\n{}:\n".format(group)
+                    text += "\n{} {}:\n".format(group_by, group)
 
                 txt, rc = self.generate_report(col_infos, grecords, right_align_num_cols, uppercase_hdr_cols, truncate_with_ellipses, 
                     col_space, duration_col_names, time_col_names, significance, max_fixed_length, precision)
@@ -790,6 +928,79 @@ class ReportBuilder():
 
         return text, row_count
 
+    def create_mean_records_core(self, records, mean_cols, group_by):
+        values = {}
+
+        for col in mean_cols:
+            values[col] = []
+            for record in records:
+                if col in record:
+                    value = record[col]
+                    if value is not None:
+                        values[col].append(float(value))
+
+        mean_records = []
+        using_single_record = True
+        
+        if using_single_record:
+            mr = {}
+            mr["item_count"] = len(records)
+            if group_by:
+                mr[group_by] = record[group_by]
+
+            for col in mean_cols:
+                data = values[col]
+
+                mean_val = np.mean(data) if data else None
+                std_err = np.std(data) / np.sqrt(len(data)) if data else None
+
+                col_name = col + "-mean" 
+                mr[col_name] = mean_val
+
+                err_col_name = col + "-err"
+                mr[err_col_name] = std_err
+
+            mean_records.append(mr)
+
+        else:
+            for col in mean_cols:
+                data = values[col]
+                mr = {}
+                mr["column"] = self.default_user_name(col)
+                mr["mean"] = np.mean(data) if data else None
+                mr["stderr"] = np.std(data) / np.sqrt(len(data)) if data else None
+
+                if group_by:
+                    mr[group_by] = record[group_by]
+
+                mean_records.append(mr)
+
+        return mean_records
+
+    def create_mean_records(self, user_cols, records, mean_cols, group_by, sort_by, reverse, first, last):
+
+        if group_by:
+            # calc mean over each groups of records
+            grouped_records = self.group_by(records, group_by)
+            mean_records = []
+            for i, (group, grecords) in enumerate(grouped_records.items()):
+                recs = self.create_mean_records_core(grecords, mean_cols, group_by)
+                mean_records += recs
+
+        else:
+            # calc mean over all records
+            mean_records = self.create_mean_records_core(grecords, mean_cols, group_by)
+
+        # apply sort, reverse, first, last
+        self.sort_records(mean_records, sort_by, reverse)
+
+        if first:
+            mean_records = mean_records[:first]
+        elif last:
+            mean_records = mean_records[-last:]
+        
+        return mean_records
+
     def should_highlight_row(self, highlight_exp, rd):
         should = False
 
@@ -799,6 +1010,26 @@ class ReportBuilder():
 
         return should
 
+    def add_dynamic_cols(self, records, user_cols):
+
+        for record in records:
+            for col in user_cols:
+                if col in record and record[col] is None:
+
+                    if col == "queued":
+                        start = utils.safe_value(record, "created")
+                        start_value = utils.get_time_from_arrow_str(start)
+                        value = time.time() - start_value
+                        record[col] = value
+
+                    elif col == "duration":
+                        start = utils.safe_value(record, "started")
+                        start_value = utils.get_time_from_arrow_str(start)
+                        value = time.time() - start_value
+                        record[col] = value
+
+        return records
+
     def format_duration(self, value, col, record):
         # value = str(datetime.timedelta(seconds=value))
         # index = value.find(".")
@@ -883,7 +1114,8 @@ class ReportBuilder():
             col_width = col_info["col_width"]
             col_name = col_info["user_name"].upper() if uppercase_hdr_cols else col_info["user_name"]
             
-            right_align = right_align_num_cols and (col_info["is_numeric"] or user_fmt) or \
+            #right_align = right_align_num_cols and (col_info["is_numeric"] or user_fmt) or \
+            right_align = right_align_num_cols and (col_info["is_numeric"]) or \
                 col_info["name"] in duration_col_names
 
             if truncate_with_ellipses and len(col_name) > col_width:
@@ -903,7 +1135,12 @@ class ReportBuilder():
             text += pc_utils.NORMAL
 
         header_line = text
-        text += "\n\n"
+
+        if len(records) > 1:
+            text += "\n\n"
+        else:
+            text += "\n"
+
         row_count = 0
 
         # process VALUE ROWS
@@ -928,6 +1165,10 @@ class ReportBuilder():
                         text += " " * col_space
 
                     user_fmt = col_info["user_fmt"] 
+                    if user_fmt == "{:s}":
+                        # don't need this anymore (was used to remove column width restriction)
+                        user_fmt = None
+
                     col = col_info["name"]
                     value_type = col_info["value_type"]
 
@@ -937,6 +1178,9 @@ class ReportBuilder():
                     col_width = col_info["col_width"]
                     align = ">" if right_align else "<"
 
+                    if col == "hp_set":
+                        dummy = 33
+
                     if not col in record:
                         # not all records define all columns
                         str_fmt = "{:" + align + str(col_width)  + "." + str(col_width) + "s}"
@@ -945,7 +1189,10 @@ class ReportBuilder():
                         value = record[col]
 
                         # special handling for int values that were logged as floats
-                        if isinstance(value, float) and value_type is int:
+                        # if isinstance(value, float) and value_type is int:
+                        #     value = int(value)
+                        if isinstance(value, str) and value.isnumeric():
+                            # handle cases where int was logged incorrectly as a string
                             value = int(value)
 
                         #console.print("col=", col, ", value=", value, ", type(value)=", type(value))
@@ -962,7 +1209,7 @@ class ReportBuilder():
                             # user provided explict format for this column
                             if "$" in user_fmt:
                                 # custom XT formatting
-                                value = self.xt_custom_format(user_fmt, value)
+                                value, align = self.xt_custom_format(user_fmt, value)
                             else:
                                 value = user_fmt.format(value)   
 
@@ -971,6 +1218,7 @@ class ReportBuilder():
                             #value = value if value else ""
                             value = "" if value is None else value
                             text += str_fmt.format(value)
+
                         elif isinstance(value, float):
 
                             # default (smart) FLOAT formatting
@@ -1002,6 +1250,9 @@ class ReportBuilder():
                 console.print("Exception formatting col={}: {}".format(col, ex))
                 console.print("  Exception record: {}".format(record))
 
+                # debug
+                raise ex
+
             if highlighted:
                 text += pc_utils.NORMAL
 
