@@ -21,7 +21,7 @@ from xtlib.helpers import file_helper
 from xtlib.helpers.feedbackParts import feedback as fb
 
 run_info_props = {"_id": 1, "app_name": 1, "box_name": 1, "compute": 1, "create_time": 1, "description": 1, "exper_name": 1,
-    "from_ip": 1, "from_computer_name": 1, "is_child": 1, "is_parent": 1, "is_outer": 1, "job_id": 1, 
+    "from_ip": 1, "from_computer_name": 1, "hp_set": 1, "is_child": 1, "is_parent": 1, "is_outer": 1, "job_id": 1, 
     "node_index": 1, "path": 1, "repeat": 1, "script": 1, "run_index": 1, "run_name": 1, "run_num": 1, "run_guid": 1,
     "search_style": 1, "search_type": 1, "service_type": 1, "sku": 1, "username": 1, "ws_name": 1, 
     "xt_build": 1, "xt_version": 1, "cluster": 1, "vc":1, "service_run_id": 1}
@@ -185,8 +185,8 @@ def remove_run_stats(dd: dict):
 user_to_actual = {"box": "box_name", "created": "create_time", "child": "is_child", "cluster": "cluster",
     "description": "description", "experiment": "exper_name", "exit_code": "exit_code", 
     "from_host": "from_computer_name", "from_ip": "from_ip", 
-    "guid": "run_guid", "job": "job_id", "last_time": "last_time",
-    "node": "node_index", "outer": "is_outer", "parent": "is_parent", "path": "path", 
+    "guid": "run_guid", "hp_set": "hp_set", "job": "job_id", "last_time": "last_time",
+    "node_index": "node_index", "outer": "is_outer", "parent": "is_parent", "path": "path", 
     "pool": "pool", "repeat": "repeat", "restarts": "restarts", "run": "run_name", "run_index": "run_index", 
     "run_num": "run_num", "script": "script", "search": "search_type", "search_style": "search_style", 
     "service_type": "service_type", "sku": "sku", "started": "start_time", "status": "status", "target": "compute", 
@@ -198,7 +198,7 @@ user_to_actual = {"box": "box_name", "created": "create_time", "child": "is_chil
 
     # run stats
     "ended": "end_time", "started": "start_time", "duration": "run_duration", "queued": "queue_duration",
-    "db_retries": "db_retries", "storage_retries": "storage_retries",
+    "db_retries": "db_retries", "storage_retries": "storage_retries", "end_id": "end_id",
 
         # special info (not a simple property)
         "metric_names": "metric_names",
@@ -214,16 +214,18 @@ std_cols_desc = {
     "db_retries": "the total number of database retries performed by the runs of this node",
     "description": "the user specified description associated with the run",
     "duration": "how long the run has been executing",
+    "end_id": "a unique serial number assigned to each run when it has ended",
     "ended": "when the run completed",
     "experiment": "the name of the experiment associated with the run",
     "exit_code": "the integer value returned by the run",
     "from_host": "the name of the computer that submitted the run",
     "from_ip": "the IP address of the computer that submitted the run",
     "guid": "a string that uniquely identifies the run",
+    "hp_set": "a string that describes the hyperparameter search values used for the run",
     "job": "the job id that the run was part of",
     "last_time": "the time of the most recent operation associated with the run",
     "metric_names": "list of metrics names, ordered by their reporting",
-    "node": "the 0-based node index of the run's box",
+    "node_index": "the 0-based node index of the run's box",
     "outer": "indicates this run is not a child run",
     "parent": "indicates that the run spawned child runs",
     "path": "the full path of the run's target script or executable file",
@@ -336,9 +338,11 @@ def build_filter_part(fd, args, arg_name, store_name):
 
 def build_run_filter_dict(store, ws_name, run_list, user_to_actual, builder, args):
     fd = {}
-    option_filters = ["job", "experiment", "target", "service_type", "box", "status", "parent", "child", "outer", "username"]  
+    option_filters = ["job", "experiment", "target", "service_type", "box", "status", "parent", "child", 
+        "outer", "username", "node_index"]  
 
     explict_options = qfe.get_explicit_options()
+    explict_options = {key.replace("-", "_"): value for key,value in explict_options.items()}
 
     # # parent is an exception - its default=0 needs to be filtered on
     # if "parent" not in explict_options:
@@ -414,7 +418,7 @@ def get_different_cols(records):
     return list(diff_cols)
 
 def get_filtered_sorted_limit_runs(store, config, show_gathering, col_dict=None, preserve_order=False, 
-        col_names_are_external=True, flatten_records=True, args=None):
+        col_names_are_external=True, flatten_records=True, user_columns=None, args=None):
     # JIT import (workaround cyclic imports)
     from xtlib.report_builder import ReportBuilder   
 
@@ -452,6 +456,11 @@ def get_filtered_sorted_limit_runs(store, config, show_gathering, col_dict=None,
     filter_dict = build_run_filter_dict(store, workspace, run_list, user_to_actual=user_to_actual, 
         builder=builder, args=args)
 
+    aux_filter = utils.safe_value(args, "aux_filter")
+    if aux_filter:
+        # merge filters together
+        filter_dict = {**filter_dict, **aux_filter}
+
     # if show_gathering:
     #     console.print("gathering run data...", flush=True)
 
@@ -459,9 +468,9 @@ def get_filtered_sorted_limit_runs(store, config, show_gathering, col_dict=None,
     need_alive = highight_exp == "$alive"
 
     # get the db records for the matching RUNS
-    records, using_default_last, last = builder.get_db_records(db, filter_dict, workspace, which="runs", 
+    records, limiter, limiter_value = builder.get_db_records(db, filter_dict, workspace, which="runs", 
         actual_to_user=actual_to_user, col_dict=col_dict, col_names_are_external=col_names_are_external, 
-        flatten_records=flatten_records, need_alive=need_alive, args=args)
+        flatten_records=flatten_records, need_alive=need_alive, user_columns=user_columns, args=args)
 
     # remove key info from _id in records
     store_utils.simplify_records_id(records)
@@ -474,7 +483,7 @@ def get_filtered_sorted_limit_runs(store, config, show_gathering, col_dict=None,
 
     console.diag("end of: get_filtered_sorted_limit_runs")
 
-    return records, using_default_last, user_to_actual, available, builder, last, std_cols_desc
+    return records, limiter, user_to_actual, available, builder, limiter_value, std_cols_desc
 
 def order_runs_by_user_list(runs, user_list):
     new_records = list(user_list)
@@ -494,9 +503,12 @@ def list_runs(store, config, args, compare=False):
     if compare:
         args["columns"] = ["job", "run", "hparams.*"]
 
-    records, using_default_last, user_to_actual, available, builder, last, std_cols_desc = \
+    report_type = "run-reports"
+    user_columns = utils.get_user_columns(args["columns"], report_type, config)
+
+    records, limiter, user_to_actual, available, builder, limiter_value, std_cols_desc = \
         get_filtered_sorted_limit_runs(store, config, show_gathering=True, 
-            col_names_are_external=True, flatten_records=True, args=args)
+            col_names_are_external=True, flatten_records=True, user_columns=user_columns, args=args)
 
     if available:
         std_cols = list(user_to_actual.keys())
@@ -516,12 +528,19 @@ def list_runs(store, config, args, compare=False):
             args["columns"] = diff_cols 
 
         #avail_list = list(user_to_actual.keys())
-        lines, row_count, was_exported = builder.build_report(records, report_type="run-reports", args=args)
+        lines, row_count, was_exported = builder.build_report(records, user_columns=user_columns, report_type=report_type, args=args)
 
         store_name = config.get("store")
         workspace = args["workspace"]
         console.print("runs on {}/{}:".format(store_name, workspace))
 
+        # print common hyperparameters on top of report
+        if utils.safe_value(args, "common") and builder.hp_common_set:
+            console.print("\nCommon hyperparameters:")
+            for hp,value in builder.hp_common_set.items():
+                console.print("  {}: {}".format(hp, value))
+            console.print("")
+
         if was_exported:
             console.print("")
 
@@ -536,10 +555,13 @@ def list_runs(store, config, args, compare=False):
                     console.print(line)
 
                 if row_count > 1:
-                    if using_default_last:
-                        console.print("total runs listed: {} (defaulted to --last={})".format(row_count, last))
+                    
+                    if limiter:
+                        avail_count = store.database.avail_row_count
+                        avail_text = "available: {:}, ".format(avail_count) if avail_count is not None else ""
+                        console.print("total runs listed: {:} ({}limited by --{}={:})".format(row_count, avail_text, limiter, limiter_value))
                     else:
-                        console.print("total runs listed: {}".format(row_count))
+                        console.print("total runs listed: {:}".format(row_count))
             else:
                 console.print("no matching runs found")
 
