@@ -36,7 +36,7 @@ class RunInfo():
         self.cmd_parts = cmd_parts
         self.run_script = run_script
         self.repeat = repeat
-        self.is_parent = bool(repeat)
+        self.is_parent = parent_name is None
         self.repeats_remaining = repeat if context.repeats_remaining is None else context.repeats_remaining 
         self.context = context
         self.status = status    
@@ -117,6 +117,7 @@ class RunInfo():
                 except BaseException as ex:
                     # if stream is closed unexpectly, treat as non-fatal error and just log
                     logger.exception("Error in process_run_output, ex={}".format(ex))
+                    print("Error in process_run_output, ex={}".format(ex))
 
     def get_core_properties(self):
         dd = {"run_name": self.run_name, "workspace": self.workspace, "cmd_parts": self.cmd_parts, "run_script": self.run_script, 
