@@ -484,6 +484,19 @@ class Runner():
         for_windows = self.backend.is_windows
         login_cmd = None
 
+        timeout = args["docker_pull_timeout"]
+        if timeout:
+            # validate timeout (e.g., "30s" or "5d")
+            if timeout.isnumeric():
+                # will default to seconds
+                pass
+            else:
+                unit = timeout[-1]
+                if not unit in 'smhd':
+                    raise Exception("illegal duration specified for general.docker-pull-timeout (try: 30m)")
+                if " " in timeout:
+                    raise Exception("illegal duration specified for general.docker-pull-timeout (try: 30m)")
+
         docker_image, login_server, docker_registry = self.config.get_docker_info(target, docker_name, required=False)
         creds_required = utils.make_numeric_if_possible( utils.safe_value(docker_registry, "login") )
 
@@ -542,20 +555,27 @@ class Runner():
         #options += " --cap-add SYS_ADMIN --device /dev/fuse --security-opt apparmor:unconfined"
         options += " --privileged"
 
+        # append user options
+        other_options = args["docker_other_options"]
+        if other_options:
+            options += " " + other_options
+
         # currently, we don't support running in a windows-based container
         docker_is_windows = False
         args["docker_is_windows"] = docker_is_windows
+        args["full_docker_image"] = full_image
 
         pass_args = "%*" if for_windows else "$*"
         sh_prefix = "bash --login"
 
         if for_windows:
             # launching docker from windows
-            docker_cmd = "{}docker run {} {} {} {} {}/{} {}".format(sudo, options, mappings, full_image, sh_prefix, dest_src_dir, constants.FN_INNER_SH, pass_args)
+            docker_cmd = "{}docker run {} {} {} {} {}/{} {}".format(sudo, mappings, options, full_image, sh_prefix, dest_src_dir, constants.FN_INNER_SH, pass_args)
         else:
             # launching docker from linux
             # we set DOCKER_RUN just before this command is executed (see backend_base.py)
-            docker_cmd = "{}$DOCKER_RUN {} {} {} {} {}/{} {}".format(sudo, options, mappings, full_image, sh_prefix, dest_src_dir, constants.FN_INNER_SH, pass_args)
+            docker_cmd = "{}$DOCKER_RUN {} {} {} {} {}/{} {}".format(sudo, mappings, options, full_image, sh_prefix, dest_src_dir, constants.FN_INNER_SH, pass_args)
+            #docker_cmd = "{}docker run --gpus all {} {} {} {} {}/{} {}".format(sudo, options, mappings, full_image, sh_prefix, dest_src_dir, constants.FN_INNER_SH, pass_args)
 
         # store for use by backend_base.wrap_user_cmds()
         args["docker_cmd"] = docker_cmd
@@ -687,7 +707,9 @@ class Runner():
         args["bootstrap_dir"] = bootstrap_dir
 
         # parse arguments before paths are removed
-        job_hparams = self.build_job_hparams(cmd_parts)
+        # we no longer support this since it writes numeric hyperparameters as string which on SQL must always be strings
+        # and having numeric hyperparameters as strings messes up HP search algorithms like DGD
+        #job_hparams = self.build_job_hparams(cmd_parts)
 
         if fake_submit:
             script_dir = snapshot_dir
@@ -821,6 +843,7 @@ class Runner():
             args["model_local"] = model_local
 
         # ADJUST CMDS: this allows backend to write scripts to snapshot dir, if needed, as a way of adjusting/wrapping run commands
+        args["job_id"] = job_id
         self.backend.adjust_run_commands(job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, args=args)
 
         # upload CODE from snapshot_dir
@@ -908,6 +931,7 @@ class Runner():
                 "job_guid": job_guid, 
                 "job_secret": job_secret, 
                 "node_count": node_count, 
+                #"maximize_metric": args["maximize_metric"],      # not yet in DB/job_stats
                 "primary_metric": args["primary_metric"], 
                 "run_count": total_run_count, 
                 "repeat": repeat_count, 
@@ -918,6 +942,9 @@ class Runner():
                 "ws_name": workspace, 
                 "xt_cmd": xt_cmd, 
 
+                # cannot add this now (since current design cannot add columns dynamically)
+                "sleep_on_exit": args["sleep_on_exit"],
+
                 # embedded READONLY props (2)
                 "pool_info": compute_def, 
                 "service_job_info": service_job_info, 
@@ -930,6 +957,7 @@ class Runner():
                 "running_nodes": 0, 
                 "running_runs": 0, 
                 "started": utils.get_arrow_time_str(), 
+                "next_run_index": 0,
 
                 # SERVICE_INFO: embedded READONLY props BY NODE (2)
                 "runs_by_box": runs_by_box, 
@@ -940,7 +968,7 @@ class Runner():
                 "secrets_by_node": secrets_by_node,  
 
                 # job-detectable HPARAMS
-                "hparams": job_hparams,
+                #"hparams": job_hparams,
 
                 # TAGS
                 "tags": tag_dict,
@@ -1372,6 +1400,7 @@ class Runner():
         runs = args["runs"]
         max_runs = args["max_runs"]
         node_repeat = args["node_repeat"]
+        runs_per_set = args["runs_per_set"]
 
         # USER MULTI CMDS
         result = self.read_user_multi_commands(using_hp, run_script, cmd_parts, args)
@@ -1419,6 +1448,15 @@ class Runner():
             cmds = new_cmds
             run_count = len(cmds)
 
+        elif using_hp and search_style == "grid" and runs_per_set and runs is None:
+            # commands (presumably from search_type=grid) are repeated N times (with no run/node assoication)
+            new_cmds = []
+            for cmd in cmds:
+                new_cmds += [cmd]*runs_per_set
+            
+            cmds = new_cmds
+            run_count = len(cmds)
+
         if show_run_report:
             console.print()   
             dr = " (dry-run)" if dry_run else ""
