@@ -532,17 +532,20 @@ class StoreV2():
         return self.helper.does_run_exist(ws_name, run_name)
 
     def get_job_id_of_run(self, ws_name, run_name):
-        job_id = None
-        records = self.get_run_log(ws_name, run_name)
-        if len(records) > 0:
-            first = records[0]
-            if "event" in first and first["event"] == "created":
-                dd = first["data"]
-                if "job_id" in dd:
-                    job_id = dd["job_id"]
+        id = store_utils.make_id(ws_name, run_name)
+        record = self.database.get_info_for_runs(ws_name, {"_id": id}, {"job_id": 1})
+        job_id = utils.safe_cursor_value(record, "job_id")
 
         return job_id
 
+    def get_node_id_of_run(self, ws_name, run_name):
+        id = store_utils.make_id(ws_name, run_name)
+        record = self.database.get_info_for_runs(ws_name, {"_id": id}, {"node_index": 1})
+        node_index = utils.safe_cursor_value(record, "node_index")
+        node_id = utils.node_id(node_index)
+
+        return node_id
+
     def _create_next_run_directory(self, ws_name, is_parent):
 
         # is this a legacy workspace?
@@ -604,7 +607,7 @@ class StoreV2():
     def start_child_run(self, ws_name, parent_run_name, exper_name=None, cmd_line_args=None, all_args=None, description=None, 
             from_ip=None, from_host=None, app_name=None, repeat=None, box_name="local", job_id=None, pool=None, 
             node_index=None, username=None, aggregate_dest=None, path=None, child_name=None, compute=None, 
-            service_type=None, search_type=None, sku=None, search_style=None, run_index=None):
+            service_type=None, search_type=None, sku=None, search_style=None, run_index=None, hp_set=None):
         '''
         This is usually called from the XT COMPUTE box, so we CANNOT get from_ip and from_computer_name from the local machine.
         '''
@@ -623,7 +626,7 @@ class StoreV2():
         self.start_run_core(ws_name, child_name, exper_name=exper_name, description=description, username=username,
             box_name=box_name, app_name=app_name, repeat=None, is_parent=False, job_id=job_id, pool=pool, node_index=node_index,
             aggregate_dest=aggregate_dest, path=path, aml_run_id=None, compute=compute, service_type=service_type, search_type=search_type,
-            sku=sku, search_style=search_style, is_child=True, run_index=run_index)           
+            sku=sku, search_style=search_style, is_child=True, run_index=run_index, hp_set=hp_set)           
 
         # finally, add a "child_created" record to the parent
         self.log_run_event(ws_name, parent_run_name, "child_created", {"child_name": child_name}, job_id=job_id)
@@ -631,7 +634,7 @@ class StoreV2():
     def start_run_core(self, ws_name, run_name, exper_name=None, description=None, username=None,
             box_name=None, app_name=None, repeat=None, is_parent=False, job_id=None, pool=None, node_index=None,
             aggregate_dest="none", path=None, aml_run_id=None, compute=None, service_type=None, search_type=None,
-            sku=None, search_style=None, is_child=False, run_index=None, tag_dict=None):
+            sku=None, search_style=None, is_child=False, run_index=None, tag_dict=None, hp_set=None):
 
         console.diag("start_run: after create_next_run_directory")
 
@@ -672,6 +675,7 @@ class StoreV2():
             "is_outer": True, 
             "is_parent": is_parent, 
             "job_id": job_id, 
+            "hp_set": hp_set,
             "node_index": node_index, 
             "path": path, 
             "repeat": repeat, 
@@ -1055,11 +1059,11 @@ class StoreV2():
         rf = self.run_files(ws_name, run_name, use_blobs=True, job_id=job_id)
         return rf.upload_file(run_fn, source_fn)
 
-    def upload_files_to_run(self, ws_name, run_name, run_folder, source_wildcard, exclude_dirs_and_files=[]):
+    def upload_files_to_run(self, ws_name, run_name, run_folder, source_wildcard, exclude_dirs_and_files=[], job_id=None):
         '''upload the local files specified by 'source_wildcard' to the run folder 'run_folder'.
         '''
         #return self.helper.upload_files_to_run(ws_name, run_name, run_folder, source_wildcard, exclude_dirs)
-        rf = self.run_files(ws_name, run_name, use_blobs=True, job_id=self.job_id)
+        rf = self.run_files(ws_name, run_name, use_blobs=True, job_id=job_id)
         return rf.upload_files(run_folder, source_wildcard, exclude_dirs_and_files=exclude_dirs_and_files)
 
     def read_log_records(self, ws_name, run_name):
@@ -1123,23 +1127,23 @@ class StoreV2():
         rf = self.run_files(ws_name, run_name, use_blobs=True)
         return rf.get_filenames(run_wildcard, full_paths=full_paths)
         
-    def delete_run_file(self, ws_name, run_name, filename):
+    def delete_run_file(self, ws_name, run_name, filename, job_id=None):
         '''delete the run file specified by 'filename'.
         '''
-        rf = self.run_files(ws_name, run_name, use_blobs=True)
+        rf = self.run_files(ws_name, run_name, use_blobs=True, job_id=job_id)
         return rf.delete_file(filename)
 
-    def delete_run_subfolder_and_files(self, ws_name, run_name, subfolder_name):
+    def delete_run_subfolder_and_files(self, ws_name, run_name, subfolder_name, job_id=None):
         '''delete all files under the specified folder_name.
         '''
-        rf = self.run_files(ws_name, run_name, use_blobs=True)
+        rf = self.run_files(ws_name, run_name, use_blobs=True, job_id=job_id)
         return rf.delete_files(subfolder_name + "/**")
 
-    def does_run_file_exist(self, ws_name, run_name, run_fn):
+    def does_run_file_exist(self, ws_name, run_name, run_fn, job_id=None):
         '''return True if the specified run file 'run_fn' exists.
         '''
         #return self.helper.does_run_file_exist(ws_name, run_name, run_fn)
-        rf = self.run_files(ws_name, run_name, use_blobs=True)
+        rf = self.run_files(ws_name, run_name, use_blobs=True, job_id=job_id)
         return rf.does_file_exist(run_fn)
 
     #---- JOBS ----
@@ -1207,12 +1211,15 @@ class StoreV2():
 
     #---- JOB FILES ----
 
-    def create_job_file(self, ws_name, job_name, job_path, text):
+    def create_job_file(self, ws_name, job_name, job_path, text, fail_if_exists=False, etag=None):
         '''create a job file specified by 'job_path' from the text 'text'.
         '''
         #self.helper.create_job_file(job_name, job_path, text)
         jf = self.job_files(ws_name, job_name, use_blobs=True)
-        return jf.create_file(job_path, text)
+        return jf.create_file(job_path, text, fail_if_exists=fail_if_exists, etag=etag)
+
+    def get_current_blob_etag(self):
+        return self.helper.provider.current_blob_etag
 
     def append_job_file(self, ws_name, job_name, job_path, text):
         '''append text 'text' to the job file 'job_path'.
