@@ -5,8 +5,11 @@
 # store-azure-blob21: Azure Blob Storage API (based on azure-storage-blob==2.1.0)
 import os
 import re
+import time
+from pytz import NonExistentTimeError
 import requests
 import logging
+import numpy as np
 from interface import implements
 
 # # debug
@@ -27,11 +30,12 @@ logger = logging.getLogger(__name__)
 MAX_PATH = 259 if os.name == "nt" else 4096
 
 class AzureBlobStore21(implements(StoreInterface)):
+
     def __init__(self, storage_creds, max_retries=25, pool_connections=25):
         self.storage_id = storage_creds["name"]
         self.storage_key = storage_creds["key"]
         self.pool_connections = utils.safe_value(storage_creds, "pool_connections", pool_connections)
-
+        self.current_blob_etag = None
         self.max_retries = max_retries    
 
         self.reset_connection()
@@ -170,12 +174,16 @@ class AzureBlobStore21(implements(StoreInterface)):
 
         return self.bs.exists(container, blob_path)
 
-    def create_blob(self, container, blob_path, text, fail_if_exists=False):
+    def create_blob(self, container, blob_path, text, fail_if_exists=False, etag=None):
         self._container_check(container, create_if_needed=True)
 
-        ifn = "*" if fail_if_exists else None
+        if etag is not None:
+            result = self.bs.create_blob_from_text(container, blob_path, text, if_match=etag)
+        else:
+            ifn = "*" if fail_if_exists else None
+            result = self.bs.create_blob_from_text(container, blob_path, text, if_none_match=ifn)
 
-        return self.bs.create_blob_from_text(container, blob_path, text, if_none_match=ifn)
+        return result
 
     def create_blob_from_path(self, container, blob_path, source_fn, progress_callback=None):
         self._container_check(container, create_if_needed=True)
@@ -195,7 +203,7 @@ class AzureBlobStore21(implements(StoreInterface)):
             return self.append_bs.append_blob_from_text(container, blob_path, text)
 
         ''' 
-        Appends text to a normal blob blob by reading and then rewriting the entire blob.
+        Appends text to a normal block blob by reading and then rewriting the entire blob.
         Correctly handles concurrency/race conditions.
         Recommended for lots of small items (like 10,000 run names).
 
@@ -299,6 +307,10 @@ class AzureBlobStore21(implements(StoreInterface)):
         text = ""
         # azure storage bug workaround: avoid RETRY errors for 0-length blob 
         blob = self.bs.get_blob_properties(container, blob_path)
+
+        # save etag where caller can retrieve it, if needed
+        self.current_blob_etag = blob.properties.etag
+
         if blob.properties.content_length:
             try:
                 blob = self.bs.get_blob_to_text(container, blob_path)
