@@ -196,6 +196,10 @@ def make_retry_func(max_retries=8, reset_func=None):
             # do not retry - fail now
             return None
 
+        if "The specified blob already exists" in str_exception:
+            # do not retry - fail now
+            raise Exception("The specified blob already exists")
+
         tb_lines = get_stack_lines()
         run_errors.record_run_error("storage", str_exception, None, tb_lines)
 
@@ -751,3 +755,40 @@ def tag_dict_from_list(tag_list):
                 td[tag] = ""
     return td
 
+def shell_time_str_to_secs(text):
+    # convert "3s" or "5m" or 6h" to seconds
+    if text[-1].isdigit():
+        # if units not specified, assume seconds
+        value = int(text)
+    else:
+        value = int(text[:-1])
+        units = text[-1]
+
+        if units == "s":
+            pass
+        elif units == "m":
+            value *= 60
+        elif units == "h":
+            value *= 60*60
+        elif units == "d":
+            value *= 60*60*60
+        else:
+            errors.ConfigError("shell time has unrecognized unit: {}".format(text))
+
+    return value
+
+def get_user_columns(columns_arg, report_name, config):
+    if columns_arg:
+        # name of a named-columns entry?
+        columns = None
+        if len(columns_arg) == 1:
+            named_col = columns_arg[0]
+            columns = config.get("named-columns", named_col)
+
+        if not columns:
+            # the actual column name(s)
+            columns = columns_arg
+    else:
+        columns = config.get(report_name, "columns")
+
+    return columns
\ No newline at end of file
