@@ -17,6 +17,7 @@ from xtlib import qfe
 from xtlib import utils
 from xtlib import errors
 from xtlib import constants
+from xtlib import file_utils
 
 from xtlib.console import console
 from xtlib.helpers.feedbackParts import feedback
@@ -45,7 +46,7 @@ def get_fn_local_config(args):
                 # is there an XT confile file in the script dir?
                 fnx = os.path.join(os.path.dirname(part), constants.CONFIG_FN)
                 if os.path.exists(fnx):
-                    fn = os.path.relpath(fnx, ".")
+                    fn = file_utils.rel_path(fnx, ".")
                     if os.path.dirname(fn):
                         # only print if its not the current directory
                         console.print("using script config file: {}".format(fn))
@@ -64,6 +65,8 @@ def get_fn_local_config(args):
                 else:
                     np = subs[1]
                 if np:
+                    if not os.path.exists(np):
+                        errors.config_error("--config file not found: {}".format(np))
                     return np
 
         else:
