@@ -92,7 +92,7 @@ class XTVault():
 
         # use normal Key Value
         from azure.keyvault.secrets import SecretClient
-        from azure.identity import DefaultAzureCredential
+        #from azure.identity import DefaultAzureCredential
 
         if authentication == "auto":
             authentication = "browser" if pc_utils.has_gui() else "device-code"
